import aiohttp
import requests

from time import time
from typing import Any
from ujson import loads
from base64 import b64decode
from functools import reduce

sync_session = requests.Session()
session = aiohttp.ClientSession()


def generate_device_sync() -> str:
    return sync_session.get("https://ed-generators.herokuapp.com/device").text
    

async def generate_device(data: str = None) -> str:
    return await (await session.get("https://ed-generators.herokuapp.com/device" + f"?data={data}" if data else "")).text()
    

async def generate_signature(data: Any) -> str:
    return await (await session.post("https://ed-generators.herokuapp.com/signature", data=data)).text()


def get_timers(size: int) -> list[dict[str, int]]:
    return tuple(map(lambda _: {"start": int(time()), "end": int(time() + 300)}, range(size)))


def decode_sid(sid: str) -> dict:
    args = (lambda a, e: a.replace(*e), ("-+", "_/"), sid+"="*(-len(sid) % 4))
    return loads(b64decode(reduce(*args).encode())[1:-20].decode())


def sid_to_uid(sid: str) -> str:
    return decode_sid(sid)["2"]


def sid_to_ip_address(sid: str) -> str:
    return decode_sid(sid)["4"]



# thanks Slimakoi#6422

class UnsupportedService(Exception):
    """
    - **API Code** : 100
    - **API Message** : Unsupported service. Your client may be out of date. Please update it to the latest version.
    - **API String** : ``Unknown String``
    """
    def __init__(*args, **kwargs):
        Exception.__init__(*args, **kwargs)


class FileTooLarge(Exception):
    """
    - **API Code** : 102
    - **API Message** : ``Unknown Message``
    - **API String** : API_STD_ERR_ENTITY_TOO_LARGE_RAW
    """
    def __init__(*args, **kwargs):
        Exception.__init__(*args, **kwargs)


class InvalidRequest(Exception):
    """
    - **API Code** : 103, 104
    - **API Message** : Invalid Request. Please update to the latest version. If the problem continues, please contact us.
    - **API String** : ``Unknown String``
    """
    def __init__(*args, **kwargs):
        Exception.__init__(*args, **kwargs)


class InvalidSession(Exception):
    """
    - **API Code** : 105
    - **API Message** : ``Unknown Message``
    - **API String** : ``Unknown String``
    """
    def __init__(*args, **kwargs):
        Exception.__init__(*args, **kwargs)


class AccessDenied(Exception):
    """
    - **API Code** : 106
    - **API Message** : Access denied.
    - **API String** : ``Unknown String``
    """
    def __init__(*args, **kwargs):
        Exception.__init__(*args, **kwargs)


class UnexistentData(Exception):
    """
    - **API Code** : 107
    - **API Message** : The requested data does not exist.
    - **API String** : ``Unknown String``
    """
    def __init__(*args, **kwargs):
        Exception.__init__(*args, **kwargs)


class ActionNotAllowed(Exception):
    """
    - **API Code** : 110
    - **API Message** : Action not allowed.
    - **API String** : ``Unknown String``
    """
    def __init__(*args, **kwargs):
        Exception.__init__(*args, **kwargs)


class ServiceUnderMaintenance(Exception):
    """
    - **API Code** : 111
    - **API Message** : Sorry, this service is under maintenance. Please check back later.
    - **API String** : ``Unknown String``
    """
    def __init__(*args, **kwargs):
        Exception.__init__(*args, **kwargs)


class MessageNeeded(Exception):
    """
    - **API Code** : 113
    - **API Message** : Be more specific, please.
    - **API String** : ``Unknown String``
    """
    def __init__(*args, **kwargs):
        Exception.__init__(*args, **kwargs)


class InvalidAccountOrPassword(Exception):
    """
    - **API Code** : 200
    - **API Message** : ``Unknown Message``
    - **API String** : ``Unknown String``
    """
    def __init__(*args, **kwargs):
        Exception.__init__(*args, **kwargs)


class AccountDisabled(Exception):
    """
    - **API Code** : 210
    - **API Message** : This account is disabled.
    - **API String** : AUTH_DISABLED_ACCOUNT
    """
    def __init__(*args, **kwargs):
        Exception.__init__(*args, **kwargs)


class InvalidEmail(Exception):
    """
    - **API Code** : 213
    - **API Message** : Invalid email address.
    - **API String** : API_ERR_EMAIL
    """
    def __init__(*args, **kwargs):
        Exception.__init__(*args, **kwargs)


class InvalidPassword(Exception):
    """
    - **API Code** : 214
    - **API Message** : Invalid password. Password must be 6 characters or more and contain no spaces.
    - **API String** : API_ERR_PASSWORD
    """
    def __init__(*args, **kwargs):
        Exception.__init__(*args, **kwargs)


class EmailAlreadyTaken(Exception):
    """
    - **API Code** : 215
    - **API Message** : Hey this email ``X`` has been registered already. You can try to log in with the email or edit the email.
    - **API String** : API_ERR_EMAIL_TAKEN
    """
    def __init__(*args, **kwargs):
        Exception.__init__(*args, **kwargs)


class UnsupportedEmail(Exception):
    """
    - **API Code** : 215
    - **API Message** : This email address is not supported.
    - **API String** : API_ERR_EMAIL_TAKEN
    """
    def __init__(*args, **kwargs):
        Exception.__init__(*args, **kwargs)


class AccountDoesntExist(Exception):
    """
    - **API Code** : 216
    - **API Message** : ``Unknown Message``
    - **API String** : AUTH_ACCOUNT_NOT_EXISTS
    """
    def __init__(*args, **kwargs):
        Exception.__init__(*args, **kwargs)


class InvalidDevice(Exception):
    """
    - **API Code** : 218
    - **API Message** : Error! Your device is currently not supported, or the app is out of date. Please update to the latest version.
    - **API String** : ``Unknown String``
    """
    def __init__(*args, **kwargs):
        Exception.__init__(*args, **kwargs)


class AccountLimitReached(Exception):
    """
    - **API Code** : 219
    - **API Message** : A maximum of 3 accounts can be created from this device. If you forget your password, please reset it.
    - **API String** : ``Unknown String``
    """
    def __init__(*args, **kwargs):
        Exception.__init__(*args, **kwargs)


class TooManyRequests(Exception):
    """
    - **API Code** : 219
    - **API Message** : Too many requests. Try again later.
    - **API String** : ``Unknown String``
    """
    def __init__(*args, **kwargs):
        Exception.__init__(*args, **kwargs)


class CantFollowYourself(Exception):
    """
    - **API Code** : 221
    - **API Message** : ``Unknown Message``
    - **API String** : ``Unknown String``
    """
    def __init__(*args, **kwargs):
        Exception.__init__(*args, **kwargs)


class UserUnavailable(Exception):
    """
    - **API Code** : 225
    - **API Message** : This user is unavailable.
    - **API String** : ``Unknown String``
    """
    def __init__(*args, **kwargs):
        Exception.__init__(*args, **kwargs)


class YouAreBanned(Exception):
    """
    - **API Code** : 229
    - **API Message** : You are banned.
    - **API String** : ``Unknown String``
    """
    def __init__(*args, **kwargs):
        Exception.__init__(*args, **kwargs)


class UserNotMemberOfCommunity(Exception):
    """
    - **API Code** : 230
    - **API Message** : You have to join this Community first.
    - **API String** : API_ERR_USER_NOT_IN_COMMUNITY
    """
    def __init__(*args, **kwargs):
        Exception.__init__(*args, **kwargs)


class RequestRejected(Exception):
    """
    - **API Code** : 235
    - **API Message** : Request rejected. You have been temporarily muted (read only mode) because you have received a strike. To learn more, please check the Help Center.
    - **API String** : ``Unknown String``
    """
    def __init__(*args, **kwargs):
        Exception.__init__(*args, **kwargs)


class ActivateAccount(Exception):
    """
    - **API Code** : 238
    - **API Message** : Please activate your account first. Check your email, including your spam folder.
    - **API String** : ``Unknown String``
    """
    def __init__(*args, **kwargs):
        Exception.__init__(*args, **kwargs)


class CantLeaveCommunity(Exception):
    """
    - **API Code** : 239
    - **API Message** : Sorry, you can not do this before transferring your Agent status to another member.
    - **API String** : ``Unknown String``
    """
    def __init__(*args, **kwargs):
        Exception.__init__(*args, **kwargs)


class ReachedTitleLength(Exception):
    """
    - **API Code** : 240
    - **API Message** : Sorry, the max length of member's title is limited to 20.
    - **API String** : ``Unknown String``
    """
    def __init__(*args, **kwargs):
        Exception.__init__(*args, **kwargs)


class EmailFlaggedAsSpam(Exception):
    """
    - **API Code** : 241
    - **API Message** : This email provider has been flagged for use in spamming.
    - **API String** : ``Unknown String``
    """
    def __init__(*args, **kwargs):
        Exception.__init__(*args, **kwargs)


class AccountDeleted(Exception):
    """
    - **API Code** : 246
    - **API Message** : ``Unknown Message``
    - **API String** : AUTH_RECOVERABLE_DELETED_ACCOUNT
    """
    def __init__(*args, **kwargs):
        Exception.__init__(*args, **kwargs)


class API_ERR_EMAIL_NO_PASSWORD(Exception):
    """
    - **API Code** : 251
    - **API Message** : ``Unknown Message``
    - **API String** : API_ERR_EMAIL_NO_PASSWORD
    """
    def __init__(*args, **kwargs):
        Exception.__init__(*args, **kwargs)


class API_ERR_COMMUNITY_USER_CREATED_COMMUNITIES_VERIFY(Exception):
    """
    - **API Code** : 257
    - **API Message** : ``Unknown Message``
    - **API String** : API_ERR_COMMUNITY_USER_CREATED_COMMUNITIES_VERIFY
    """
    def __init__(*args, **kwargs):
        Exception.__init__(*args, **kwargs)


class ReachedMaxTitles(Exception):
    """
    - **API Code** : 262
    - **API Message** : You can only add up to 20 Titles. Please choose the most relevant ones.
    - **API String** : ``Unknown String``
    """
    def __init__(*args, **kwargs):
        Exception.__init__(*args, **kwargs)


class VerificationRequired(Exception):
    """
    - **API Code** : 270
    - **API Message** : Verification Required.
    - **API String** : API_ERR_NEED_TWO_FACTOR_AUTHENTICATION
    """
    def __init__(*args, **kwargs):
        Exception.__init__(*args, **kwargs)


class API_ERR_INVALID_AUTH_NEW_DEVICE_LINK(Exception):
    """
    - **API Code** : 271
    - **API Message** : ``Unknown Message``
    - **API String** : API_ERR_INVALID_AUTH_NEW_DEVICE_LINK
    """
    def __init__(*args, **kwargs):
        Exception.__init__(*args, **kwargs)


class CommandCooldown(Exception):
    """
    - **API Code** : 291
    - **API Message** : Whoa there! You've done too much too quickly. Take a break and try again later.
    - **API String** : ``Unknown String``
    """
    def __init__(*args, **kwargs):
        Exception.__init__(*args, **kwargs)


class UserBannedByTeamAmino(Exception):
    """
    - **API Code** : 293
    - **API Message** : Sorry, this user has been banned by Team Amino.
    - **API String** : ``Unknown String``
    """
    def __init__(*args, **kwargs):
        Exception.__init__(*args, **kwargs)


class BadImage(Exception):
    """
    - **API Code** : 300
    - **API Message** : ``Unknown Message``
    - **API String** : ``Unknown String``
    """
    def __init__(*args, **kwargs):
        Exception.__init__(*args, **kwargs)


class InvalidThemepack(Exception):
    """
    - **API Code** : 313
    - **API Message** : ``Unknown Message``
    - **API String** : ``Unknown String``
    """
    def __init__(*args, **kwargs):
        Exception.__init__(*args, **kwargs)


class InvalidVoiceNote(Exception):
    """
    - **API Code** : 314
    - **API Message** : ``Unknown Message``
    - **API String** : ``Unknown String``
    """
    def __init__(*args, **kwargs):
        Exception.__init__(*args, **kwargs)


class RequestedNoLongerExists(Exception):
    """
    - **API Code** : 500, 700, 1600
    - **API Message** : Sorry, the requested data no longer exists. Try refreshing the view.
    - **API String** : ``Unknown String``
    """
    def __init__(*args, **kwargs):
        Exception.__init__(*args, **kwargs)


class PageRepostedTooRecently(Exception):
    """
    - **API Code** : 503
    - **API Message** : Sorry, you have reported this page too recently.
    - **API String** : ``Unknown String``
    """
    def __init__(*args, **kwargs):
        Exception.__init__(*args, **kwargs)


class InsufficientLevel(Exception):
    """
    - **API Code** : 551
    - **API Message** : This post type is restricted to members with a level ``X`` ranking and above.
    - **API String** : ``Unknown String``
    """
    def __init__(*args, **kwargs):
        Exception.__init__(*args, **kwargs)


class WallCommentingDisabled(Exception):
    """
    - **API Code** : 702
    - **API Message** : This member has disabled commenting on their wall.
    - **API String** : ``Unknown String``
    """
    def __init__(*args, **kwargs):
        Exception.__init__(*args, **kwargs)


class CommunityNoLongerExists(Exception):
    """
    - **API Code** : 801
    - **API Message** : This Community no longer exists.
    - **API String** : ``Unknown String``
    """
    def __init__(*args, **kwargs):
        Exception.__init__(*args, **kwargs)


class InvalidCodeOrLink(Exception):
    """
    - **API Code** : 802
    - **API Message** : Sorry, this code or link is invalid.
    - **API String** : ``Unknown String``
    """
    def __init__(*args, **kwargs):
        Exception.__init__(*args, **kwargs)


class CommunityNameAlreadyTaken(Exception):
    """
    - **API Code** : 805
    - **API Message** : ``Unknown Message``
    - **API String** : ``Unknown String``
    """
    def __init__(*args, **kwargs):
        Exception.__init__(*args, **kwargs)


class CommunityCreateLimitReached(Exception):
    """
    - **API Code** : 806
    - **API Message** : ``Unknown Message``
    - **API String** : API_ERR_COMMUNITY_USER_CREATED_COMMUNITIES_EXCEED_QUOTA
    """
    def __init__(*args, **kwargs):
        Exception.__init__(*args, **kwargs)


class CommunityDisabled(Exception):
    """
    - **API Code** : 814
    - **API Message** : This Community is disabled.
    - **API String** : ``Unknown String``
    """
    def __init__(*args, **kwargs):
        Exception.__init__(*args, **kwargs)


class CommunityDeleted(Exception):
    """
    - **API Code** : 833
    - **API Message** : This Community has been deleted.
    - **API String** : ``Unknown String``
    """
    def __init__(*args, **kwargs):
        Exception.__init__(*args, **kwargs)


class ReachedMaxCategories(Exception):
    """
    - **API Code** : 1002
    - **API Message** : Sorry, you can create up to 100 categories.
    - **API String** : ``Unknown String``
    """
    def __init__(*args, **kwargs):
        Exception.__init__(*args, **kwargs)


class DuplicatePollOption(Exception):
    """
    - **API Code** : 1501
    - **API Message** : Sorry, you have duplicate poll options.
    - **API String** : ``Unknown String``
    """
    def __init__(*args, **kwargs):
        Exception.__init__(*args, **kwargs)


class ReachedMaxPollOptions(Exception):
    """
    - **API Code** : 1507
    - **API Message** : Sorry, you can only join or add up to 5 of your items per poll.
    - **API String** : ``Unknown String``
    """
    def __init__(*args, **kwargs):
        Exception.__init__(*args, **kwargs)


class TooManyChats(Exception):
    """
    - **API Code** : 1602
    - **API Message** : Sorry, you can only have up to 1000 chat sessions.
    - **API String** : ``Unknown String``
    """

    def __init__(*args, **kwargs):
        Exception.__init__(*args, **kwargs)


class ChatFull(Exception):
    """
    - **API Code** : 1605
    - **API Message** : ``Unknown Message``
    - **API String** : ``Unknown String``
    """
    def __init__(*args, **kwargs):
        Exception.__init__(*args, **kwargs)


class TooManyInviteUsers(Exception):
    """
    - **API Code** : 1606
    - **API Message** : Sorry, you can only invite up to 999 people.
    - **API String** : ``Unknown String``
    """
    def __init__(*args, **kwargs):
        Exception.__init__(*args, **kwargs)


class ChatInvitesDisabled(Exception):
    """
    - **API Code** : 1611
    - **API Message** : This user has disabled chat invite requests.
    - **API String** : ``Unknown String``
    """
    def __init__(*args, **kwargs):
        Exception.__init__(*args, **kwargs)


class RemovedFromChat(Exception):
    """
    - **API Code** : 1612
    - **API Message** : You've been removed from this chatroom.
    - **API String** : ``Unknown String``
    """
    def __init__(*args, **kwargs):
        Exception.__init__(*args, **kwargs)


class UserNotJoined(Exception):
    """
    - **API Code** : 1613
    - **API Message** : Sorry, this user has not joined.
    - **API String** : ``Unknown String``
    """
    def __init__(*args, **kwargs):
        Exception.__init__(*args, **kwargs)


class API_ERR_CHAT_VVCHAT_NO_MORE_REPUTATIONS(Exception):
    """
    - **API Code** : 1627
    - **API Message** : ``Unknown Message``
    - **API String** : API_ERR_CHAT_VVCHAT_NO_MORE_REPUTATIONS
    """
    def __init__(*args, **kwargs):
        Exception.__init__(*args, **kwargs)


class MemberKickedByOrganizer(Exception):
    """
    - **API Code** : 1637
    - **API Message** : This member was previously kicked by the organizer and cannot be reinvited.
    - **API String** : ``Unknown String``
    """
    def __init__(*args, **kwargs):
        Exception.__init__(*args, **kwargs)


class LevelFiveRequiredToEnableProps(Exception):
    """
    - **API Code** : 1661
    - **API Message** : ``Unknown Message``
    - **API String** : ``Unknown String``
    """
    def __init__(*args, **kwargs):
        Exception.__init__(*args, **kwargs)


class ChatViewOnly(Exception):
    """
    - **API Code** : 1663
    - **API Message** : ``Unknown Message``
    - **API String** : ``Unknown String``
    """
    def __init__(*args, **kwargs):
        Exception.__init__(*args, **kwargs)


class ChatMessageTooBig(Exception):
    """
    - **API Code** : 1664
    - **API Message** : ``Unknown Message``
    - **API String** : API_ERR_CHAT_MESSAGE_CONTENT_TOO_LONG
    """
    def __init__(*args, **kwargs):
        Exception.__init__(*args, **kwargs)


class InviteCodeNotFound(Exception):
    """
    - **API Code** : 1900
    - **API Message** : Sorry, the requested data no longer exists. Try refreshing the view.
    - **API String** : ``Unknown String``
    """
    def __init__(*args, **kwargs):
        Exception.__init__(*args, **kwargs)


class AlreadyRequestedJoinCommunity(Exception):
    """
    - **API Code** : 2001
    - **API Message** : Sorry, you have already submitted a membership request.
    - **API String** : ``Unknown String``
    """
    def __init__(*args, **kwargs):
        Exception.__init__(*args, **kwargs)


class API_ERR_PUSH_SERVER_LIMITATION_APART(Exception):
    """
    - **API Code** : 2501
    - **API Message** : ``Unknown Message``
    - **API String** : API_ERR_PUSH_SERVER_LIMITATION_APART
    """
    def __init__(*args, **kwargs):
        Exception.__init__(*args, **kwargs)


class API_ERR_PUSH_SERVER_LIMITATION_COUNT(Exception):
    """
    - **API Code** : 2502
    - **API Message** : ``Unknown Message``
    - **API String** : API_ERR_PUSH_SERVER_LIMITATION_COUNT
    """
    def __init__(*args, **kwargs):
        Exception.__init__(*args, **kwargs)


class API_ERR_PUSH_SERVER_LINK_NOT_IN_COMMUNITY(Exception):
    """
    - **API Code** : 2503
    - **API Message** : ``Unknown Message``
    - **API String** : API_ERR_PUSH_SERVER_LINK_NOT_IN_COMMUNITY
    """
    def __init__(*args, **kwargs):
        Exception.__init__(*args, **kwargs)


class API_ERR_PUSH_SERVER_LIMITATION_TIME(Exception):
    """
    - **API Code** : 2504
    - **API Message** : ``Unknown Message``
    - **API String** : API_ERR_PUSH_SERVER_LIMITATION_TIME
    """
    def __init__(*args, **kwargs):
        Exception.__init__(*args, **kwargs)


class AlreadyCheckedIn(Exception):
    """
    - **API Code** : 2601
    - **API Message** : Sorry, you can't check in any more.
    - **API String** : ``Unknown String``
    """
    def __init__(*args, **kwargs):
        Exception.__init__(*args, **kwargs)


class AlreadyUsedMonthlyRepair(Exception):
    """
    - **API Code** : 2611
    - **API Message** : ``Unknown Message``
    - **API String** : ``Unknown String``
    """
    def __init__(*args, **kwargs):
        Exception.__init__(*args, **kwargs)


class AccountAlreadyRestored(Exception):
    """
    - **API Code** : 2800
    - **API Message** : Account already restored.
    - **API String** : ``Unknown String``
    """
    def __init__(*args, **kwargs):
        Exception.__init__(*args, **kwargs)


class IncorrectVerificationCode(Exception):
    """
    - **API Code** : 3102
    - **API Message** : Incorrect verification code.
    - **API String** : ``Unknown String``
    """
    def __init__(*args, **kwargs):
        Exception.__init__(*args, **kwargs)


class NotOwnerOfChatBubble(Exception):
    """
    - **API Code** : 3905
    - **API Message** : You are not the owner of this chat bubble.
    - **API String** : ``Unknown String``
    """
    def __init__(*args, **kwargs):
        Exception.__init__(*args, **kwargs)


class NotEnoughCoins(Exception):
    """
    - **API Code** : 4300
    - **API Message** : ``Unknown Message``
    - **API String** : ``Unknown String``
    """
    def __init__(*args, **kwargs):
        Exception.__init__(*args, **kwargs)


class AlreadyPlayedLottery(Exception):
    """
    - **API Code** : 4400
    - **API Message** : You have played the maximum number of lucky draws.
    - **API String** : ``Unknown String``
    """
    def __init__(*args, **kwargs):
        Exception.__init__(*args, **kwargs)


class CannotSendCoins(Exception):
    """
    - **API Code** : 4500, 4501
    - **API Message** : ``Unknown Message``
    - **API String** : ``Unknown String``
    """
    def __init__(*args, **kwargs):
        Exception.__init__(*args, **kwargs)


class AminoIDAlreadyChanged(Exception):
    """
    - **API Code** : 6001
    - **API Message** : Amino ID cannot be changed after you set it.
    - **API String** : ``Unknown String``
    """
    def __init__(*args, **kwargs):
        Exception.__init__(*args, **kwargs)


class InvalidAminoID(Exception):
    """
    - **API Code** : 6002
    - **API Message** : Invalid Amino ID
    - **API String** : ``Unknown String``
    """
    def __init__(*args, **kwargs):
        Exception.__init__(*args, **kwargs)


class InvalidName(Exception):
    """
    - **API Code** : 99001
    - **API Message** : Sorry, the name is invalid.
    - **API String** : ``Unknown String``
    """
    def __init__(*args, **kwargs):
        Exception.__init__(*args, **kwargs)


class SpecifyType(Exception):
    """
    Raised when you need to specify the output of the command.
    """
    def __init__(*args, **kwargs):
        Exception.__init__(*args, **kwargs)


class WrongType(Exception):
    """
    Raised when you attribute the function the wrong type.
    """
    def __init__(*args, **kwargs):
        Exception.__init__(*args, **kwargs)


class UnknownResponse(Exception):
    """
    Raised when an error occurs but the reason is unknown.
    """
    def __init__(*args, **kwargs):
        Exception.__init__(*args, **kwargs)


class NotLoggedIn(Exception):
    """
    Raised when you try to make an action but you aren't logged in.
    """
    def __init__(*args, **kwargs):
        Exception.__init__(*args, **kwargs)


class NoCommunity(Exception):
    """
    Raised when you try to make an action but no community was selected.
    """
    def __init__(*args, **kwargs):
        Exception.__init__(*args, **kwargs)


class CommunityNotFound(Exception):
    """
    Raised when you search for a community but nothing is found.
    """
    def __init__(*args, **kwargs):
        Exception.__init__(*args, **kwargs)


class NoChatThread(Exception):
    """
    Raised when you try to make an action but no chat was selected.
    """
    def __init__(*args, **kwargs):
        Exception.__init__(*args, **kwargs)


class ChatRequestsBlocked(Exception):
    """
    Raised when you try to make an action but the end user has chat requests blocked.
    """
    def __init__(*args, **kwargs):
        Exception.__init__(*args, **kwargs)


class NoImageSource(Exception):
    """
    Raised when you try to make an action but no image source was selected.
    """
    def __init__(*args, **kwargs):
        Exception.__init__(*args, **kwargs)


class CannotFetchImage(Exception):
    """
    Raised when an image cannot be fetched.
    """
    def __init__(*args, **kwargs):
        Exception.__init__(*args, **kwargs)


class FailedLogin(Exception):
    """
    Raised when you try to login but it fails.
    """
    def __init__(*args, **kwargs):
        Exception.__init__(*args, **kwargs)


class AgeTooLow(Exception):
    """
    Raised when you try to configure an account but the age is too low. Minimum is 13.
    """
    def __init__(*args, **kwargs):
        Exception.__init__(*args, **kwargs)


class UnsupportedLanguage(Exception):
    """
    Raised when you try to use a language that isn't supported or exists.
    """
    def __init__(*args, **kwargs):
        Exception.__init__(*args, **kwargs)


class CommunityNeeded(Exception):
    """
    Raised when you try to execute an command but a Community needs to be specified.
    """
    def __init__(*args, **kwargs):
        Exception.__init__(*args, **kwargs)


class FlagTypeNeeded(Exception):
    """
    Raised when you try to flag a community, blog or user but a Flag Type needs to be specified.
    """
    def __init__(*args, **kwargs):
        Exception.__init__(*args, **kwargs)


class ReasonNeeded(Exception):
    """
    Raised when you try to execute an command but a Reason needs to be specified.
    """
    def __init__(*args, **kwargs):
        Exception.__init__(*args, **kwargs)


class TransferRequestNeeded(Exception):
    """
    Raised when you need to transfer host to complete the action.
    """
    def __init__(*args, **kwargs):
        Exception.__init__(*args, **kwargs)


class LibraryUpdateAvailable(Exception):
    """
    Raised when a new library update is available.
    """
    def __init__(*args, **kwargs):
        Exception.__init__(*args, **kwargs)


def CheckException(data):
    api_code = data["api:statuscode"]

    if api_code == 0:
        return
    elif api_code == 100:
        raise UnsupportedService(data)
    elif api_code == 102:
        raise FileTooLarge(data)
    elif api_code == 103 or api_code == 104:
        raise InvalidRequest(data)
    elif api_code == 105:
        raise InvalidSession(data)
    elif api_code == 106:
        raise AccessDenied(data)
    elif api_code == 107:
        raise UnexistentData(data)
    elif api_code == 110:
        raise ActionNotAllowed(data)
    elif api_code == 111:
        raise ServiceUnderMaintenance(data)
    elif api_code == 113:
        raise MessageNeeded(data)
    elif api_code == 200:
        raise InvalidAccountOrPassword(data)
    elif api_code == 201:
        raise AccountDisabled(data)
    elif api_code == 210:
        raise AccountDisabled(data)
    elif api_code == 213:
        raise InvalidEmail(data)
    elif api_code == 214:
        raise InvalidPassword(data)
    elif api_code == 215:
        raise EmailAlreadyTaken(data) and UnsupportedEmail(data)
    elif api_code == 216:
        raise AccountDoesntExist(data)
    elif api_code == 218:
        raise InvalidDevice(data)
    elif api_code == 219:
        raise AccountLimitReached(data) or TooManyRequests(data)
    elif api_code == 221:
        raise CantFollowYourself(data)
    elif api_code == 225:
        raise UserUnavailable(data)
    elif api_code == 229:
        raise YouAreBanned(data)
    elif api_code == 230:
        raise UserNotMemberOfCommunity(data)
    elif api_code == 235:
        raise RequestRejected(data)
    elif api_code == 238:
        raise ActivateAccount(data)
    elif api_code == 239:
        raise CantLeaveCommunity(data)
    elif api_code == 240:
        raise ReachedTitleLength(data)
    elif api_code == 241:
        raise EmailFlaggedAsSpam(data)
    elif api_code == 246:
        raise AccountDeleted(data)
    elif api_code == 251:
        raise API_ERR_EMAIL_NO_PASSWORD(data)
    elif api_code == 257:
        raise API_ERR_COMMUNITY_USER_CREATED_COMMUNITIES_VERIFY(data)
    elif api_code == 262:
        raise ReachedMaxTitles(data)
    elif api_code == 270:
        raise VerificationRequired(data)
    elif api_code == 271:
        raise API_ERR_INVALID_AUTH_NEW_DEVICE_LINK(data)
    elif api_code == 291:
        raise CommandCooldown(data)
    elif api_code == 293:
        raise UserBannedByTeamAmino(data)
    elif api_code == 300:
        raise BadImage(data)
    elif api_code == 313:
        raise InvalidThemepack(data)
    elif api_code == 314:
        raise InvalidVoiceNote(data)
    elif api_code == 500 or api_code == 700 or api_code == 1600:
        raise RequestedNoLongerExists(data)
    elif api_code == 503:
        raise PageRepostedTooRecently(data)
    elif api_code == 551:
        raise InsufficientLevel(data)
    elif api_code == 702:
        raise WallCommentingDisabled(data)
    elif api_code == 801:
        raise CommunityNoLongerExists(data)
    elif api_code == 802:
        raise InvalidCodeOrLink(data)
    elif api_code == 805:
        raise CommunityNameAlreadyTaken(data)
    elif api_code == 806:
        raise CommunityCreateLimitReached(data)
    elif api_code == 814:
        raise CommunityDisabled(data)
    elif api_code == 833:
        raise CommunityDeleted(data)
    elif api_code == 1002:
        raise ReachedMaxCategories(data)
    elif api_code == 1501:
        raise DuplicatePollOption(data)
    elif api_code == 1507:
        raise ReachedMaxPollOptions(data)
    elif api_code == 1602:
        raise TooManyChats(data)
    elif api_code == 1605:
        raise ChatFull(data)
    elif api_code == 1606:
        raise TooManyInviteUsers(data)
    elif api_code == 1611:
        raise ChatInvitesDisabled(data)
    elif api_code == 1612:
        raise RemovedFromChat(data)
    elif api_code == 1613:
        raise UserNotJoined(data)
    elif api_code == 1627:
        raise API_ERR_CHAT_VVCHAT_NO_MORE_REPUTATIONS(data)
    elif api_code == 1637:
        raise MemberKickedByOrganizer(data)
    elif api_code == 1661:
        raise LevelFiveRequiredToEnableProps(data)
    elif api_code == 1663:
        raise ChatViewOnly(data)
    elif api_code == 1664:
        raise ChatMessageTooBig(data)
    elif api_code == 1900:
        raise InviteCodeNotFound(data)
    elif api_code == 2001:
        raise AlreadyRequestedJoinCommunity(data)
    elif api_code == 2501:
        raise API_ERR_PUSH_SERVER_LIMITATION_APART(data)
    elif api_code == 2502:
        raise API_ERR_PUSH_SERVER_LIMITATION_COUNT(data)
    elif api_code == 2503:
        raise API_ERR_PUSH_SERVER_LINK_NOT_IN_COMMUNITY(data)
    elif api_code == 2504:
        raise API_ERR_PUSH_SERVER_LIMITATION_TIME(data)
    elif api_code == 2601:
        raise AlreadyCheckedIn(data)
    elif api_code == 2611:
        raise AlreadyUsedMonthlyRepair(data)
    elif api_code == 2800:
        raise AccountAlreadyRestored(data)
    elif api_code == 3102:
        raise IncorrectVerificationCode(data)
    elif api_code == 3905:
        raise NotOwnerOfChatBubble(data)
    elif api_code == 4300:
        raise NotEnoughCoins(data)
    elif api_code == 4400:
        raise AlreadyPlayedLottery(data)
    elif api_code == 4500:
        raise CannotSendCoins(data)
    elif api_code == 4501:
        raise CannotSendCoins(data)
    elif api_code == 6001:
        raise AminoIDAlreadyChanged(data)
    elif api_code == 6002:
        raise InvalidAminoID(data)
    elif api_code == 9901:
        raise InvalidName(data)
    else:
        raise Exception(data)



from asyncio import get_event_loop, new_event_loop
from asyncio.events import AbstractEventLoop
from base64 import b64encode
from time import time, timezone
from typing import BinaryIO, Union, List
from uuid import uuid4
from aiohttp import ClientSession, ClientTimeout
from json_minify import json_minify
from ujson import dumps, loads

from .utils.models import *
from .utils.types import *
from .utils.helpers import *
from .utils.exceptions import *
from .http_client import AminoHttpClient


class CommunityClient(AminoHttpClient):
    def __init__(self, comId: int,
        loop: Optional[AbstractEventLoop] = None,
        session: Optional[ClientSession] = None, 
        info: Optional[Community] = None,
        settings: Optional[dict] = None,
        proxy: Optional[str] = None,
        proxy_auth: Optional[str] = None
    ) -> None:
        self._session: ClientSession = session or ClientSession(
            timeout=ClientTimeout(60), json_serialize=dumps)
        self._loop: AbstractEventLoop = loop or get_event_loop()
        
        self.comId = comId
        self.headers = settings or self.headers

        self.proxy: Optional[str] = proxy
        self.proxy_auth: Optional[str] = proxy_auth
        
        self.profile: UserProfile = UserProfile(**{})
        self.info: Community = info or Community(**{})

    async def __aenter__(self) -> "CommunityClient":
        return self

    async def __aexit__(self, *args) -> None:
        await self._session.close()
    
    def __del__(self):
        self._loop.create_task(self._close_session())
    
    @property
    def event_loop(self):
        if not self._loop:
            try:
                self._loop = get_event_loop()
            except RuntimeError:
                self._loop = new_event_loop()
        return self._loop

    async def _close_session(self):
        if not self.session.closed:
            await self.session.close()
    
    async def get_invite_codes(self, status: str = "normal", start: int = 0, size: int = 25) -> List[InviteCode]:
        response = await self.get(f"/g/s-x{self.comId}/community/invitation?status={status}&start={start}&size={size}")
        return list(map(lambda o: InviteCode(**o), (await response.json())["communityInvitationList"]))

    async def get_user_info(self, userId: str) -> UserProfile:
        response = await self.get(f"/x{self.comId}/s/user-profile/{userId}")
        return UserProfile(**(await response.json())["userProfile"])

    async def generate_invite_code(self, duration: int = 0, force: bool = True):
        data = {
            "duration": duration,
            "force": force,
            "timestamp": int(time() * 1000)
        }

        response = await self.post(f"/g/s-x{self.comId}/community/invitation", data)
        return InviteCode(**(await response.json())["communityInvitation"])

    async def delete_invite_code(self, inviteId: str) -> int:
        response = await self.delete(f"/g/s-x{self.comId}/community/invitation/{inviteId}")
        return response.status
    
    async def delete_blog(self, blogId: str) -> int:
        response = await self.delete(f"/x{self.comId}/s/blog/{blogId}")
        return response.status

    async def delete_wiki(self, wikiId: str) -> int:
        response = await self.delete(f"/x{self.comId}/s/item/{wikiId}")
        return response.status
    
    async def upload_media(self, file: BinaryIO, fileType: str) -> str:
        response = await self.post(f"/x{self.comId}/s/media/upload", data=file.read(), type=fileType)
        return (await response.json())["mediaValue"]

    async def post_blog(self, title: str, content: str, imageList: list = None, captionList: list = None, 
            categoriesList: list = None, backgroundColor: str = None, fansOnly: bool = False, extensions: dict = None) -> int:
        if captionList and imageList:
            mediaList = [[100, self.upload_media(image, "image"), caption] for image, caption in zip(imageList, captionList)]

        elif imageList:
            mediaList = [[100, self.upload_media(image, "image"), None] for image in imageList]

        data = {
            "address": None,
            "content": content,
            "title": title,
            "mediaList": mediaList,
            "extensions": extensions,
            "latitude": 0,
            "longitude": 0,
            "eventSource": SourceTypes.GLOBAL_COMPOSE,
            "timestamp": int(time() * 1000)
        }

        if fansOnly: data["extensions"] = {"fansOnly": fansOnly}
        if backgroundColor: data["extensions"] = {"style": {"backgroundColor": backgroundColor}}
        if categoriesList: data["taggedBlogCategoryIdList"] = categoriesList

        response = await self.post(f"/x{self.comId}/s/blog", data)
        return response.status
    
    async def post_wiki(self, title: str, content: str, icon: str = None, imageList: list = None,
            keywords: str = None, backgroundColor: str = None, fansOnly: bool = False) -> int:

        mediaList = [[100, self.upload_media(image, "image"), None] for image in imageList]

        data = {
            "label": title,
            "content": content,
            "mediaList": mediaList,
            "eventSource": SourceTypes.GLOBAL_COMPOSE,
            "timestamp": int(time() * 1000)
        }

        if icon: data["icon"] = icon
        if keywords: data["keywords"] = keywords
        if fansOnly: data["extensions"] = {"fansOnly": fansOnly}
        if backgroundColor: data["extensions"] = {"style": {"backgroundColor": backgroundColor}}

        response = await self.post(f"/x{self.comId}/s/item", data)
        return response.status

    async def edit_blog(self, blogId: str, title: str = None, content: str = None, imageList: list = None, 
            categoriesList: list = None, backgroundColor: str = None, fansOnly: bool = False) -> int:
        mediaList = [[100, self.upload_media(image, "image"), None] for image in imageList]

        data = {
            "address": None,
            "mediaList": mediaList,
            "latitude": 0,
            "longitude": 0,
            "eventSource": "PostDetailView",
            "timestamp": int(time() * 1000)
        }

        if title: data["title"] = title
        if content: data["content"] = content
        if fansOnly: data["extensions"] = {"fansOnly": fansOnly}
        if backgroundColor: data["extensions"] = {"style": {"backgroundColor": backgroundColor}}
        if categoriesList: data["taggedBlogCategoryIdList"] = categoriesList

        response = await self.post(f"/x{self.comId}/s/blog/{blogId}", data)
        return response.status
    
    async def repost_blog(self, content: str = None, blogId: str = None, wikiId: str = None) -> int:
        if blogId:
            refObjectId, refObjectType = blogId, ObjectTypes.BLOG
        elif wikiId: 
            refObjectId, refObjectType = wikiId, ObjectTypes.ITEM
        else: raise SpecifyType()

        data = {
            "content": content,
            "refObjectId": refObjectId,
            "refObjectType": refObjectType,
            "type": 2,
            "timestamp": int(time() * 1000)
        }

        response = await self.post(f"/x{self.comId}/s/blog", data)
        return response.status
    
    async def check_in(self, tz: int = -timezone // 1000) -> CheckIn:
        data = {
            "timezone": tz,
            "timestamp": int(time() * 1000)
        }

        response = await self.post(f"/x{self.comId}/s/check-in", data)
        return CheckIn(**(await response.json()))

    async def repair_check_in(self, method: int = RepairTypes.COINS) -> int:
        data = {
            "repairMethod": method,
            "timestamp": int(time() * 1000)
        }
        response = await self.post(f"/x{self.comId}/s/check-in/repair", data)
        return response.status

    async def lottery(self, tz: int = -timezone // 1000) -> int:
        data = {
            "timezone": tz,
            "timestamp": int(time() * 1000)
        }

        response = await self.post(f"/x{self.comId}/s/check-in/lottery", data)
        return Lottery(**(await response.json())["lotteryLog"])
    
    async def edit_profile(self, nickname: str = None, content: str = None, icon: BinaryIO = None, chatRequestPrivilege: str = None, 
            imageList: list = None, captionList: list = None, backgroundImage: str = None, backgroundColor: str = None, 
            titles: list = None, colors: list = None, defaultBubbleId: str = None) -> int:
        data = {"timestamp": int(time() * 1000)}

        if captionList and imageList:
            mediaList = [[100, self.upload_media(image, "image"), caption] for image, caption in zip(imageList, captionList)]

        elif imageList:
            mediaList = [[100, self.upload_media(image, "image"), None] for image in imageList]

        if imageList or captionList and imageList:
            data["mediaList"] = mediaList

        if nickname: data["nickname"] = nickname
        if icon: data["icon"] = self.upload_media(icon, "image")
        if content: data["content"] = content

        if chatRequestPrivilege: data["extensions"] = {"privilegeOfChatInviteRequest": chatRequestPrivilege}
        if backgroundImage: data["extensions"] = {"style": {"backgroundMediaList": [[100, backgroundImage, None, None, None]]}}
        if backgroundColor: data["extensions"] = {"style": {"backgroundColor": backgroundColor}}
        if defaultBubbleId: data["extensions"] = {"defaultBubbleId": defaultBubbleId}

        if titles or colors:
            tlt = [{"title": titles, "color": colors} for titles, colors in zip(titles, colors)]
            data["extensions"] = {"customTitles": tlt}

        response = await self.post(f"/x{self.comId}/s/user-profile/{self.userId}", data)
        return response.status
    
    async def vote_poll(self, blogId: str, optionId: str) -> int:
        data = {
            "value": 1,
            "eventSource": SourceTypes.DATAIL_POST,
            "timestamp": int(time() * 1000)
        }

        response = await self.post(f"/x{self.comId}/s/blog/{blogId}/poll/option/{optionId}/vote", data)
        return response.status
    
    async def comment(self, message: str, userId: str = None, 
            blogId: str = None, wikiId: str = None, replyTo: str = None) -> int:
        data = {
            "content": message,
            "stickerId": None,
            "type": 0,
            "timestamp": int(time() * 1000)
        }

        if replyTo:
            data["respondTo"] = replyTo

        if userId:
            data["eventSource"] = SourceTypes.USER_PROFILE
            response = await self.post(f"/x{self.comId}/s/user-profile/{userId}/g-comment", data)
            return response.status

        elif blogId:
            data["eventSource"] = SourceTypes.DATAIL_POST
            response = await self.post(f"/x{self.comId}/s/blog/{blogId}/g-comment", data)
            return response.status

        elif wikiId:
            data["eventSource"] = SourceTypes.DATAIL_POST
            response = await self.post(f"/x{self.comId}/s/item/{wikiId}/g-comment", data)
            return response.status

        else: raise SpecifyType()
    
    async def delete_comment(self, commentId: str, userId: str = None, blogId: str = None, wikiId: str = None) -> int:
        if userId: url = f"/x{self.comId}/s/user-profile/{userId}/comment/{commentId}"
        elif blogId: url = f"/x{self.comId}/s/blog/{blogId}/comment/{commentId}"
        elif wikiId: url = f"/x{self.comId}/s/item/{wikiId}/comment/{commentId}"
        else: raise SpecifyType()

        response = await self.delete(url)
        return response.status
    
    async def like_blog(self, blogId: Union[str, list] = None, wikiId: str = None) -> int:
        data = {
            "value": 4,
            "timestamp": int(time() * 1000)
        }

        if blogId:
            if isinstance(blogId, str):
                data["eventSource"] = SourceTypes.USER_PROFILE
                response = await self.post(f"/x{self.comId}/s/blog/{blogId}/g-vote?cv=1.2", data)
                return response.status

            elif isinstance(blogId, list):
                data["targetIdList"] = blogId
                response = await self.post(f"/x{self.comId}/s/feed/g-vote", data)
                return response.status

            else: raise WrongType(type(blogId))

        elif wikiId:
            data["eventSource"] = SourceTypes.DATAIL_POST
            response = await self.post(f"/x{self.comId}/s/item/{wikiId}/g-vote?cv=1.2", data)
            return response.status

        else: raise SpecifyType()

    async def unlike_blog(self, blogId: str = None, wikiId: str = None) -> int:
        if blogId:
            url = f"/x{self.comId}/s/blog/{blogId}/vote?eventSource=UserProfileView"

        elif wikiId:
            url = f"/x{self.comId}/s/item/{wikiId}/vote?eventSource=PostDetailView"

        else: raise SpecifyType()

        response = await self.delete(url)
        return response.status
    
    async def like_comment(self, commentId: str, userId: str = None, blogId: str = None, wikiId: str = None) -> int:
        data = {
            "value": 4,
            "timestamp": int(time() * 1000)
        }

        if userId:
            data["eventSource"] = SourceTypes.USER_PROFILE
            response = await self.post(f"/x{self.comId}/s/user-profile/{userId}/comment/{commentId}/g-vote?cv=1.2&value=1", data)
            return response.status

        elif blogId:
            data["eventSource"] = SourceTypes.DATAIL_POST
            response = await self.post(f"/x{self.comId}/s/blog/{blogId}/comment/{commentId}/g-vote?cv=1.2&value=1", data)
            return response.status

        elif wikiId:
            data["eventSource"] = SourceTypes.DATAIL_POST
            response = await self.post(f"/x{self.comId}/s/item/{wikiId}/comment/{commentId}/g-vote?cv=1.2&value=1", data)
            return response.status

        else: raise SpecifyType()
    
    async def unlike_comment(self, commentId: str, userId: str = None, blogId: str = None, wikiId: str = None) -> int:
        if userId: 
            url = f"/g/s/user-profile/{userId}/comment/{commentId}/g-vote?eventSource=UserProfileView"

        elif blogId: 
            url = f"/g/s/blog/{blogId}/comment/{commentId}/g-vote?eventSource=PostDetailView"

        elif wikiId: 
            url = f"/g/s/item/{wikiId}/comment/{commentId}/g-vote?eventSource=PostDetailView"

        else: raise SpecifyType()

        response = await self.delete(url)
        return response.status
    
    async def upvote_comment(self, blogId: str, commentId: str) -> int:
        data = {
            "value": 1,
            "eventSource": "PostDetailView",
            "timestamp": int(time() * 1000)
        }

        response = await self.post(f"/x{self.comId}/s/blog/{blogId}/comment/{commentId}/vote?cv=1.2&value=1", data)
        return response.status

    async def downvote_comment(self, blogId: str, commentId: str) -> int:
        data = {
            "value": -1,
            "eventSource": "PostDetailView",
            "timestamp": int(time() * 1000)
        }

        response = await self.post(f"/x{self.comId}/s/blog/{blogId}/comment/{commentId}/vote?cv=1.2&value=-1", data)
        return response.status

    async def unvote_comment(self, blogId: str, commentId: str) -> int:
        response = await self.delete(f"/x{self.comId}/s/blog/{blogId}/comment/{commentId}/vote?eventSource=PostDetailView")
        return response.status
    
    async def reply_wall(self, userId: str, commentId: str, message: str) -> int:
        data = {
            "content": message,
            "stackedId": None,
            "respondTo": commentId,
            "type": 0,
            "eventSource": "UserProfileView",
            "timestamp": int(time() * 1000)
        }

        response = await self.post(f"/x{self.comId}/s/user-profile/{userId}/comment", data)
        return response.status

    async def activity_status(self, status: int) -> int:
        data = {
            "onlineStatus": status,
            "duration": 86400,
            "timestamp": int(time() * 1000)
        }

        response = await self.post(f"/x{self.comId}/s/user-profile/{self.userId}/online-status", data)
        return response.status
    
    async def check_notifications(self) -> int:
        response = await self.post(f"/x{self.comId}/s/notification/checked")
        return response.status

    async def delete_notification(self, notificationId: str) -> int:
        response = await self.delete(f"/x{self.comId}/s/notification/{notificationId}")
        return response.status

    async def clear_notifications(self) -> int:
        response = await self.delete(f"/x{self.comId}/s/notification")
        return response.status
    
    async def start_chat(self, userId: Union[str, list], message: str, title: str = None,
            content: str = None, isGlobal: bool = False, publishToGlobal: bool = False) -> Thread:
        data = {
            "title": title,
            "inviteeUids": userId if isinstance(userId, list) else [userId],
            "initialMessageContent": message,
            "content": content,
            "timestamp": int(time() * 1000)
        }

        if isGlobal: 
            data["type"] = ChatPublishTypes.IS_GLOBAL
            data["eventSource"] = SourceTypes.GLOBAL_COMPOSE
        else: data["type"] = ChatPublishTypes.OFF

        if publishToGlobal: 
            data["publishToGlobal"] = ChatPublishTypes.ON
        else: data["publishToGlobal"] = ChatPublishTypes.OFF

        response = await self.post(f"/x{self.comId}/s/chat/thread", data)
        return Thread(**(await response.json()))
    
    async def invite_to_chat(self, userId: Union[str, list], chatId: str) -> int:
        data = {
            "uids": userId if isinstance(userId, list) else [userId],
            "timestamp": int(time() * 1000)
        }

        response = await self.post(f"/x{self.comId}/s/chat/thread/{chatId}/member/invite", data)
        return response.status

    async def add_to_favorites(self, userId: str) -> int:
        response = await self.post(f"/x{self.comId}/s/user-group/quick-access/{userId}")
        return response.status
    
    async def send_coins(self, coins: int, blogId: str = None,
            chatId: str = None, objectId: str = None, transactionId: str = None) -> int:
        data = {
            "coins": coins,
            "tippingContext": {
                "transactionId": transactionId or str(uuid4())
            },
            "timestamp": int(time() * 1000)
        }

        if blogId:
            response = await self.post(f"/g/s/blog/{blogId}/tipping", data)
            return response.status
        
        elif chatId:
            response = await self.post(f"/g/s/chat/thread/{chatId}/tipping", data)
            return response.status
        
        elif objectId:
            data["objectId"] = objectId
            data["objectType"] = ObjectTypes.ITEM
            response = await self.post(f"/g/s/tipping", data)
            return response.status
        
        else: SpecifyType() 

    async def thank_tip(self, chatId: str, userId: str) -> int:
        response = await self.post(f"/x{self.comId}/s/chat/thread/{chatId}/tipping/tipped-users/{userId}/thank")
        return response.status
    
    async def follow(self, userId: Union[str, list]) -> int:
        if isinstance(userId, str):
            response = await self.post(f"/x{self.comId}/s/user-profile/{userId}/member")
            return response.status

        elif isinstance(userId, list):
            data = {"targetUidList": userId, "timestamp": int(time() * 1000)}
            response = await self.post(f"/x{self.comId}/s/user-profile/{self.userId}/joined", data)
            return response.status

        else: raise WrongType(type(userId))

    async def unfollow(self, userId: str) -> int:
        response = await self.delete(f"/x{self.comId}/s/user-profile/{self.userId}/joined/{userId}")
        return response.status
    
    async def block(self, userId: str) -> int:
        response = await self.post(f"/x{self.comId}/s/block/{userId}")
        return response.status

    async def unblock(self, userId: str) -> int:
        response = await self.delete(f"/x{self.comId}/s/block/{userId}")
        return response.status
    
    async def flag(self, reason: str, flagType: int, userId: str = None,
            blogId: str = None, wikiId: str = None) -> int:
        if reason is None: raise ReasonNeeded
        if flagType is None: raise FlagTypeNeeded

        data = {
            "flagType": flagType,
            "message": reason,
            "timestamp": int(time() * 1000)
        }

        if userId:
            data["objectId"] = userId
            data["objectType"] = ObjectTypes.USER

        elif blogId:
            data["objectId"] = blogId
            data["objectType"] = ObjectTypes.BLOG

        elif wikiId:
            data["objectId"] = wikiId
            data["objectType"] = ObjectTypes.ITEM

        else: raise SpecifyType

        response = await self.post(f"/x{self.comId}/s/flag", data)
        return response.status
    
    async def send_message(self, chatId: str, message: str = None, type: int = 0, replyTo: str = None, 
            mentions: list = None, embedId: str = None, embedType: int = None, embedLink: str = None, 
            embedTitle: str = None, embedContent: str = None, embedImage: Union[BinaryIO, str] = None) -> Message:

        message = message.replace("<$", "‎‏")
        message = message.replace("$>", "‬‭")
        mentions = [{"uid": uid} for uid in mentions if mentions] if mentions else None

        if embedImage:
            if isinstance(embedImage, str):
                embedImage = [[100, embedImage, None]]
            elif isinstance(embedImage, BinaryIO):
                embedImage = [[100, await self.upload_media(embedImage, "image"), None]]
            else:
                raise SpecifyType()

        data = {
            "type": type,
            "content": message,
            "clientRefId": int(time() / 10 % 1000000000),
            "attachedObject": {
                "objectId": embedId,     # ID object (user, blog, and other)
                "objectType": embedType, # ObjectTypes
                "link": embedLink,       # ObjectLink
                "title": embedTitle,     # Embed title
                "content": embedContent, # Embed message
                "mediaList": embedImage  # ObjectPreview
            },
            "extensions": {"mentionedArray": mentions},
            "timestamp": int(time() * 1000)
        }

        if replyTo:
            data["replyMessageId"] = replyTo

        response = await self.post(f"/x{self.comId}/s/chat/thread/{chatId}/message", data)
        return Message(**(await response.json())["message"])
    
    async def send_image(self, chatId: str, file: BinaryIO) -> Message:
        data = {
            "type": 0,
            "mediaType": 100,
            "mediaUhqEnabled": True,
            "clientRefId": int(time() / 10 % 1000000000),
            "timestamp": int(time() * 1000)
        }

        data["mediaUploadValueContentType"] = FileTypes.IMAGE
        data["mediaUploadValue"] = b64encode(file.read()).decode()

        response = await self.post(f"/x{self.comId}/s/chat/thread/{chatId}/message", data)
        return Message(**(await response.json())["message"])
    
    async def send_audio(self, chatId: str, file: BinaryIO) -> Message:
        data = {
            "type": 2,
            "mediaType": 110,
            "clientRefId": int(time() / 10 % 1000000000),
            "timestamp": int(time() * 1000)
        }

        data["mediaUploadValueContentType"] = FileTypes.AUDIO
        data["mediaUploadValue"] = b64encode(file.read()).decode()

        response = await self.post(f"/x{self.comId}/s/chat/thread/{chatId}/message", data)
        return Message(**(await response.json())["message"])
    
    async def send_sticker(self, chatId: str, stickerId: str) -> Message:
        data = {
            "type": 3,
            "stickerId": stickerId,
            "clientRefId": int(time() / 10 % 1000000000),
            "timestamp": int(time() * 1000)
        }

        response = await self.post(f"/x{self.comId}/s/chat/thread/{chatId}/message", data)
        return Message(**(await response.json())["message"])
    
    async def delete_message(self, chatId: str, messageId: str, asStaff: bool = False, reason: str = None) -> int:
        data = {
            "adminOpName": 102,
            "adminOpNote": {"content": reason},
            "timestamp": int(time() * 1000)
        }

        if asStaff and reason:
            data["adminOpNote"] = {"content": reason}

        if not asStaff:
            response = await self.delete(f"/x{self.comId}/s/chat/thread/{chatId}/message/{messageId}")
            return response.status

        response = await self.post(f"/x{self.comId}/s/chat/thread/{chatId}/message/{messageId}/admin", data)
        return response.status
    
    async def mark_as_read(self, chatId: str, messageId: str):
        data = {
            "messageId": messageId,
            "timestamp": int(time() * 1000)
        }

        response = await self.post(f"/x{self.comId}/s/chat/thread/{chatId}/mark-as-read", data)
        return response.status
    
    async def edit_chat(self, chatId: str, doNotDisturb: bool = None, pinChat: bool = None, 
            title: str = None, icon: str = None, backgroundImage: BinaryIO = None, content: str = None, 
            announcement: str = None, coHosts: list = None, keywords: list = None, pinAnnouncement: bool = None, 
            publishToGlobal: bool = None, canTip: bool = None, viewOnly: bool = None, canInvite: bool = None, fansOnly: bool = None) -> int:
        data = {"timestamp": int(time() * 1000)}
        responses = []

        if title: data["title"] = title
        if content: data["content"] = content
        if icon: data["icon"] = icon
        if keywords: data["keywords"] = keywords
        if announcement: data["extensions"] = {"announcement": announcement}
        if pinAnnouncement: data["extensions"] = {"pinAnnouncement": pinAnnouncement}
        if fansOnly: data["extensions"] = {"fansOnly": fansOnly}

        if publishToGlobal: data["publishToGlobal"] = 0
        if not publishToGlobal: data["publishToGlobal"] = 1

        if coHosts:
            data = {"uidList": coHosts, "timestamp": int(time() * 1000)}
            response = await self.post(f"/x{self.comId}/s/chat/thread/{chatId}/co-host", data)
            responses.append(response.status)

        if doNotDisturb is True:
            data = {"alertOption": 2, "timestamp": int(time() * 1000)}
            response = await self.post(f"/x{self.comId}/s/chat/thread/{chatId}/member/{self.userId}/alert", data)
            responses.append(response.status)

        if doNotDisturb is False:
            data = {"alertOption": 1, "timestamp": int(time() * 1000)}
            response = await self.post(f"/x{self.comId}/s/chat/thread/{chatId}/member/{self.userId}/alert", data)
            responses.append(response.status)

        if backgroundImage:
            data = {"media": [100, await self.upload_media(backgroundImage, "image"), None], "timestamp": int(time() * 1000)}
            response = await self.post(f"/x{self.comId}/s/chat/thread/{chatId}/member/{self.userId}/background", data)
            responses.append(response.status)
        
        if pinChat is True: responses.append((await self.post(f"/x{self.comId}/s/chat/thread/{chatId}/pin", data)).status)
        if pinChat is False: responses.append((await self.post(f"/x{self.comId}/s/chat/thread/{chatId}/unpin", data)).status)

        if viewOnly is True: responses.append((await self.post(f"/x{self.comId}/s/chat/thread/{chatId}/view-only/enable", data)).status)
        if viewOnly is False: responses.append((await self.post(f"/x{self.comId}/s/chat/thread/{chatId}/view-only/disable", data)).status)

        if canInvite is True: responses.append((await self.post(f"/x{self.comId}/s/chat/thread/{chatId}/members-can-invite/enable", data)).status)
        if canInvite is False: responses.append((await self.post(f"/x{self.comId}/s/chat/thread/{chatId}/members-can-invite/disable", data)).status)

        if canTip is True: responses.append((await self.post(f"/x{self.comId}/s/chat/thread/{chatId}/tipping-perm-status/enable", data)).status)
        if canTip is False: responses.append((await self.post(f"/x{self.comId}/s/chat/thread/{chatId}/tipping-perm-status/disable", data)).status)

        responses.append((await self.post(f"/x{self.comId}/s/chat/thread/{chatId}", data)).status)
        return int(sum(responses) / len(responses))
    
    async def transfer_organizer(self, chatId: str, userIds: list) -> int:
        data = {
            "uidList": userIds,
            "timestamp": int(time() * 1000)
        }

        response = await self.post(f"/x{self.comId}/s/chat/thread/{chatId}/transfer-organizer", data)
        return response.status

    async def accept_organizer(self, chatId: str, requestId: str) -> int:
        response = await self.post(f"/x{self.comId}/s/chat/thread/{chatId}/transfer-organizer/{requestId}/accept")
        return response.status

    async def kick(self, userId: str, chatId: str, allowRejoin: bool = True) -> int:
        if allowRejoin: allowRejoin = 1
        if not allowRejoin: allowRejoin = 0

        response = await self.delete(f"/x{self.comId}/s/chat/thread/{chatId}/member/{userId}?allowRejoin={allowRejoin}")
        return response.status

    async def join_chat(self, chatId: str) -> int:
        response = await self.post(f"/x{self.comId}/s/chat/thread/{chatId}/member/{self.userId}")
        return response.status

    async def leave_chat(self, chatId: str) -> int:
        response = await self.delete(f"/x{self.comId}/s/chat/thread/{chatId}/member/{self.userId}")
        return response.status
        
    async def delete_chat(self, chatId: str) -> int:
        response = await self.delete(f"/x{self.comId}/s/chat/thread/{chatId}")
        return response.status
    
    async def subscribe(self, userId: str, autoRenew: str = False, transactionId: str = None) -> int:
        data = {
            "paymentContext": {
                "transactionId": transactionId or str(uuid4()),
                "isAutoRenew": autoRenew
            },
            "timestamp": int(time() * 1000)
        }

        response = await self.post(f"/x{self.comId}/s/influencer/{userId}/subscribe", data)
        return response.status

    async def promotion(self, noticeId: str, type: str = "accept") -> int:
        response = await self.post(f"/x{self.comId}/s/notice/{noticeId}/{type}")
        return response.status

    async def play_quiz_raw(self, quizId: str, quizAnswerList: list, quizMode: int = 0) -> int:
        data = {
            "mode": quizMode,
            "quizAnswerList": quizAnswerList,
            "timestamp": int(time() * 1000)
        }

        response = await self.post(f"/x{self.comId}/s/blog/{quizId}/quiz/result", data)
        return response.status
    
    async def play_quiz(self, quizId: str, questionIdsList: list, answerIdsList: list, quizMode: int = 0) -> int:
        quizAnswerList = []

        for question, answer in zip(questionIdsList, answerIdsList):
            part = {
                "optIdList": [answer],
                "quizQuestionId": question,
                "timeSpent": 0.0
            }

            quizAnswerList.append(loads(part))

        data = {
            "mode": quizMode,
            "quizAnswerList": quizAnswerList,
            "timestamp": int(time() * 1000)
        }

        response = await self.post(f"/x{self.comId}/s/blog/{quizId}/quiz/result", data)
        return response.status
    
    async def vc_permission(self, chatId: str, permission: int) -> int:
        data = {
            "vvChatJoinType": permission,
            "timestamp": int(time() * 1000)
        }

        response = await self.post(f"/x{self.comId}/s/chat/thread/{chatId}/vvchat-permission", data)
        return response.status

    async def get_vc_reputation_info(self, chatId: str) -> VcReputation:
        response = await self.get(f"/x{self.comId}/s/chat/thread/{chatId}/avchat-reputation")
        return VcReputation(**(await response.json()))

    async def claim_vc_reputation(self, chatId: str) -> VcReputation:
        response = await self.post(f"/x{self.comId}/s/chat/thread/{chatId}/avchat-reputation")
        return VcReputation(**(await response.json()))
    
    async def get_all_users(self, type: str = "recent", start: int = 0, size: int = 25) -> List[UserProfile]:
        response = await self.get(f"/x{self.comId}/s/user-profile?type={type}&start={start}&size={size}")
        return list(map(lambda o: UserProfile(**o), (await response.json())["userProfileList"]))

    async def get_online_favorite_users(self, start: int = 0, size: int = 25) -> List[UserProfile]:
        response = await self.get(f"/x{self.comId}/s/user-group/quick-access?type=online&start={start}&size={size}")
        return list(map(lambda o: UserProfile(**o), (await response.json())["userProfileList"]))

    async def get_user_following(self, userId: str, start: int = 0, size: int = 25) -> List[UserProfile]:
        response = await self.get(f"/x{self.comId}/s/user-profile/{userId}/joined?start={start}&size={size}")
        return list(map(lambda o: UserProfile(**o), (await response.json())["userProfileList"]))

    async def get_user_followers(self, userId: str, start: int = 0, size: int = 25) -> List[UserProfile]:
        response = await self.get(f"/x{self.comId}/s/user-profile/{userId}/member?start={start}&size={size}")
        return list(map(lambda o: UserProfile(**o), (await response.json())["userProfileList"]))

    async def get_user_checkins(self, userId: str) -> List[CheckIn]:
        response = await self.get(f"/x{self.comId}/s/check-in/stats/{userId}?timezone={-timezone // 1000}")
        return list(map(lambda o: CheckIn(**o), (await response.json())))

    async def get_user_blogs(self, userId: str, start: int = 0, size: int = 25) -> List[Blog]:
        response = await self.get(f"/x{self.comId}/s/blog?type=user&q={userId}&start={start}&size={size}")
        return list(map(lambda o: Blog(**o), (await response.json())["blogList"]))

    async def get_user_wikis(self, userId: str, start: int = 0, size: int = 25) -> List[Wiki]:
        response = await self.get(f"/x{self.comId}/s/item?type=user-all&start={start}&size={size}&cv=1.2&uid={userId}")
        return list(map(lambda o: Wiki(**o), (await response.json())["itemList"]))

    async def get_user_achievements(self, userId: str) -> Achievement:
        response = await self.get(f"/x{self.comId}/s/user-profile/{userId}/achievements")
        return Achievement(**(await response.json())["achievements"])

    async def get_influencer_fans(self, userId: str, start: int = 0, size: int = 25) -> List[UserProfile]:
        response = await self.get(f"/x{self.comId}/s/influencer/{userId}/fans?start={start}&size={size}")
        return list(map(lambda o: UserProfile(**o), (await response.json())))

    async def get_blocked_users(self, start: int = 0, size: int = 25) -> List[UserProfile]:
        response = await self.get(f"/x{self.comId}/s/block?start={start}&size={size}")
        return list(map(lambda o: UserProfile(**o), (await response.json())["userProfileList"]))

    async def get_blocker_users(self, start: int = 0, size: int = 25) -> List[str]:
        response = await self.get(f"/x{self.comId}/s/block?start={start}&size={size}")
        return loads(await response.text())["blockerUidList"]

    async def search_users(self, nickname: str, start: int = 0, size: int = 25) -> List[UserProfile]:
        response = await self.get(f"/x{self.comId}/s/user-profile?type=name&q={nickname}&start={start}&size={size}")
        return list(map(lambda o: UserProfile(**o), (await response.json())["userProfileList"]))

    async def get_saved_blogs(self, start: int = 0, size: int = 25) -> List[Bookmark]:
        response = await self.get(f"/x{self.comId}/s/bookmark?start={start}&size={size}")
        return list(map(lambda o: Bookmark(**o), (await response.json())["bookmarkList"]))
    
    async def get_leaderboard_info(self, type: int, start: int = 0, size: int = 25) -> List[UserProfile]:
        response = await self.get(f"/g/s-x{self.comId}/community/leaderboard?rankingType={type}&start={start}&size={size}")
        return list(map(lambda o: UserProfile(**o), (await response.json())["userProfileList"]))
    
    async def get_tipped_users(self, blogId: str = None, wikiId: str = None, 
            fileId: str = None, chatId: str = None, start: int = 0, size: int = 25) -> List[TippedUserSummary]:
        if blogId:
            url = f"/x{self.comId}/s/blog/{blogId}/tipping/tipped-users-summary?start={start}&size={size}"

        elif wikiId: 
            url = f"/x{self.comId}/s/item/{wikiId}/tipping/tipped-users-summary?start={start}&size={size}"

        elif chatId: 
            url = f"/x{self.comId}/s/chat/thread/{chatId}/tipping/tipped-users-summary?start={start}&size={size}"

        elif fileId: 
            url = f"/x{self.comId}/s/shared-folder/files/{fileId}/tipping/tipped-users-summary?start={start}&size={size}"

        else: raise SpecifyType()

        response = await self.get(url)
        return list(map(lambda o: TippedUserSummary(**o), await response.json()))
    
    async def get_chat_threads(self, start: int = 0, size: int = 25) -> List[Thread]:
        response = await self.get(f"/x{self.comId}/s/chat/thread?type=joined-me&start={start}&size={size}")
        return list(map(lambda o: Thread(**o), (await response.json())["threadList"]))

    async def get_public_chat_threads(self, type: str = "recommended", start: int = 0, size: int = 25) -> List[Thread]:
        response = await self.get(f"/x{self.comId}/s/chat/thread?type=public-all&filterType={type}&start={start}&size={size}")
        return list(map(lambda o: Thread(**o), (await response.json())["threadList"]))

    async def get_chat_thread(self, chatId: str) -> Thread:
        response = await self.get(f"/x{self.comId}/s/chat/thread/{chatId}")
        return Thread(loads(await response.json())["thread"])

    async def get_chat_messages(self, chatId: str, size: int = 25, pageToken: str = None) -> Message:
        if not pageToken: params = f"v=2&pagingType=t&size={size}"
        else: params = f"v=2&pagingType=t&pageToken={pageToken}&size={size}"

        response = await self.get(f"/x{self.comId}/s/chat/thread/{chatId}/message?{params}")
        return list(map(lambda o, p: Message(**o, **p), (data := await response.json())["messageList"], data["paging"]))

    async def get_message_info(self, chatId: str, messageId: str) -> Message:
        response = await self.get(f"/x{self.comId}/s/chat/thread/{chatId}/message/{messageId}")
        return Message(**(await response.json())["message"])
    
    async def get_blog_info(self, blogId: str = None, wikiId: str = None, fileId: str = None) -> Union[Wiki, Blog]:
        if blogId:
            response = await self.get(f"/x{self.comId}/s/blog/{blogId}")
            return Blog(**(await response.json()))

        elif wikiId:
            response = await self.get(f"/x{self.comId}/s/item/{wikiId}")
            return Wiki(**(await response.json()))

        elif fileId:
            response = await self.get(f"/x{self.comId}/s/shared-folder/files/{fileId}")
            return Wiki(**(await response.json())["file"])

        else: raise SpecifyType()
    
    async def get_blog_comments(self, blogId: str = None, wikiId: str = None, 
            fileId: str = None, sorting: str = "newest", start: int = 0, size: int = 25) -> List[Comment]:
        if blogId:
            url = f"/x{self.comId}/s/blog/{blogId}/comment?sort={sorting}&start={start}&size={size}"

        elif wikiId:
            url = f"/x{self.comId}/s/item/{wikiId}/comment?sort={sorting}&start={start}&size={size}"

        elif fileId:
            url = f"/x{self.comId}/s/shared-folder/files/{fileId}/comment?sort={sorting}&start={start}&size={size}"

        else: raise SpecifyType()

        response = await self.get(url)
        return list(map(lambda o: Comment(**o), (await response.json())["commentList"]))
    
    async def get_blog_categories(self, size: int = 25) -> List[BlogCategory]:
        response = await self.get(f"/x{self.comId}/s/blog-category?size={size}")
        return list(map(lambda o: BlogCategory(**o), (await response.json())["blogCategoryList"]))

    async def get_blogs_by_category(self, categoryId: str, start: int = 0, size: int = 25) -> List[Blog]:
        response = await self.get(f"/x{self.comId}/s/blog-category/{categoryId}/blog-list?start={start}&size={size}")
        return list(map(lambda o: Blog(**o), (await response.json())["blogList"]))

    async def get_quiz_rankings(self, quizId: str, start: int = 0, size: int = 25)-> QuizRanking:
        response = await self.get(f"/x{self.comId}/s/blog/{quizId}/quiz/result?start={start}&size={size}")
        return QuizRanking(**(await response.json()))
    
    async def get_wall_comments(self, userId: str, sorting: str, start: int = 0, size: int = 25) -> List[Comment]:
        response = await self.get(f"/x{self.comId}/s/user-profile/{userId}/comment?sort={sorting}&start={start}&size={size}")
        return list(map(lambda o: Comment(**o), (await response.json())["commentList"]))

    async def get_recent_blogs(self, pageToken: str = None, start: int = 0, size: int = 25) -> Blog:
        if not pageToken: params = f"v=2&pagingType=t&start={start}&size={size}"
        else: params = f"v=2&pagingType=t&pageToken={pageToken}&start={start}&size={size}"
        
        response = await self.get(f"/x{self.comId}/s/feed/blog-all?{params}")
        return list(map(lambda o, p: Blog(**o, **p), (data := await response.json())["blogList"], data["paging"]))

    async def get_chat_users(self, chatId: str, start: int = 0, size: int = 25) -> List[UserProfile]:
        response = await self.get(f"/x{self.comId}/s/chat/thread/{chatId}/member?start={start}&size={size}&type=default&cv=1.2")
        return list(map(lambda o: UserProfile(**o), (await response.json())["memberList"]))
    
    # TODO : Finish this
    async def get_notifications(self, start: int = 0, size: int = 25) -> dict:
        response = await self.get(f"/x{self.comId}/s/notification?pagingType=t&start={start}&size={size}")
        return (await response.json())["notificationList"]
    
    async def get_sticker_pack_info(self, sticker_pack_id: str) -> StickerCollection:
        response = await self.get(f"/x{self.comId}/s/sticker-collection/{sticker_pack_id}?includeStickers=true")
        return StickerCollection(**(await response.json())["stickerCollection"])

    async def get_sticker_packs(self) -> List[StickerCollection]:
        response = await self.get(f"/x{self.comId}/s/sticker-collection?includeStickers=false&type=my-active-collection")
        return list(map(lambda o: StickerCollection(**o), (await response.json())["stickerCollection"]))

    async def get_store_chat_bubbles(self, start: int = 0, size: int = 25) -> List[StoreItem]:
        response = await self.get(f"/x{self.comId}/s/store/items?sectionGroupId=chat-bubble&start={start}&size={size}")
        return list(map(lambda o: StoreItem(**o), (await response.json())["stickerCollection"]))

    async def get_store_stickers(self, start: int = 0, size: int = 25) -> List[StoreItem]:
        response = await self.get(f"/x{self.comId}/s/store/items?sectionGroupId=sticker&start={start}&size={size}")
        return list(map(lambda o: StoreItem(**o), (await response.json())["stickerCollection"]))
    
    async def get_community_stickers(self) -> List[StickerCollection]:
        response = await self.get(f"/x{self.comId}/s/sticker-collection?type=community-shared")
        return list(map(lambda o: StickerCollection(**o), (await response.json())))

    async def get_sticker_collection(self, collectionId: str) -> StickerCollection:
        response = await self.get(f"/x{self.comId}/s/sticker-collection/{collectionId}?includeStickers=true")
        return StickerCollection(**(await response.json())["stickerCollection"])

    # TODO : Finish this
    async def get_shared_folder_info(self) -> dict:
        response = await self.get(f"/x{self.comId}/s/shared-folder/stats")
        return (await response.json())["stats"]

    # TODO : Finish this
    async def get_shared_folder_files(self, type: str = "latest", start: int = 0, size: int = 25) -> dict:
        response = await self.get(f"/x{self.comId}/s/shared-folder/files?type={type}&start={start}&size={size}")
        return (await response.json())["fileList"]
    
    async def reorder_featured_users(self, userIds: list) -> int:
        data = {
            "uidList": userIds,
            "timestamp": int(time() * 1000)
        }

        response = await self.post(f"/x{self.comId}/s/user-profile/featured/reorder", data)
        return response.status

    async def get_hidden_blogs(self, start: int = 0, size: int = 25) -> Blog:
        response = await self.get(f"/x{self.comId}/s/feed/blog-disabled?start={start}&size={size}")
        return list(map(lambda o: Blog(**o), (await response.json())["blogList"]))

    async def get_featured_users(self, start: int = 0, size: int = 25) -> List[UserProfile]:
        response = await self.get(f"/x{self.comId}/s/user-profile?type=featured&start={start}&size={size}")
        return list(map(lambda o: UserProfile(**o), (await response.json())["userProfileList"])) 

    async def review_quiz_questions(self, quizId: str) -> List[Blog.QuizQuestion]:
        response = await self.get(f"/x{self.comId}/s/blog/{quizId}?action=review")
        return list(map(lambda o: Blog.QuizQuestion(**o), (await response.json())["blog"]["quizQuestionList"]))

    async def get_recent_quiz(self, start: int = 0, size: int = 25) -> Blog:
        response = await self.get(f"/x{self.comId}/s/blog?type=quizzes-recent&start={start}&size={size}")
        return list(map(lambda o: Blog(**o), (await response.json())["blogList"]))

    async def get_trending_quiz(self, start: int = 0, size: int = 25) -> Blog:
        response = await self.get(f"/x{self.comId}/s/feed/quiz-trending?start={start}&size={size}")
        return list(map(lambda o: Blog(**o), (await response.json())["blogList"]))

    async def get_best_quiz(self, start: int = 0, size: int = 25) -> Blog:
        response = await self.get(f"/x{self.comId}/s/feed/quiz-best-quizzes?start={start}&size={size}")
        return list(map(lambda o: Blog(**o), (await response.json())["blogList"]))
    
    async def moderation_history(self, userId: str = None, blogId: str = None, 
            wikiId: str = None, quizId: str = None, fileId: str = None, size: int = 25) -> List[AdminLog]:
        if userId:
            objectId = userId
            type = ObjectTypes.USER

        elif blogId:
            objectId = blogId
            type = ObjectTypes.BLOG

        elif quizId:
            objectId = quizId
            type = ObjectTypes.BLOG

        elif wikiId:
            objectId = wikiId
            type = ObjectTypes.ITEM

        elif fileId:
            objectId = fileId
            type = ObjectTypes.FOLDER_FILE

        else:
            objectId = None
            type = None

        response = await self.get(f"/x{self.comId}/s/admin/operation?pagingType=t&size={size}" \
            "?objectId={userId}&objectType={type}" if type and objectId else "")
        return list(map(lambda o: AdminLog(**o), (await response.json())["adminLogList"]))

    async def feature(self, seconds: int, userId: str = None,
            chatId: str = None, blogId: str = None, wikiId: str = None) -> int:
        data = {
            "adminOpName": 114,
            "adminOpValue": {
                "featuredDuration": seconds
            },
            "timestamp": int(time() * 1000)
        }

        if userId:
            data["adminOpValue"] = {"featuredType": FeaturedTypes.USER}
            response = await self.post(f"/x{self.comId}/s/user-profile/{userId}/admin", data)
            return response.status

        elif blogId:
            data["adminOpValue"] = {"featuredType": FeaturedTypes.BLOG}
            response = await self.post(f"/x{self.comId}/s/blog/{blogId}/admin", data)
            return response.status

        elif wikiId:
            data["adminOpValue"] = {"featuredType": FeaturedTypes.WIKI}
            response = await self.post(f"/x{self.comId}/s/item/{wikiId}/admin", data)
            return response.status

        elif chatId:
            data["adminOpValue"] = {"featuredType": FeaturedTypes.CHAT}
            response = await self.post(f"/x{self.comId}/s/chat/thread/{chatId}/admin", data)
            return response.status

        else: raise SpecifyType()

    async def unfeature(self, userId: str = None, chatId: str = None, blogId: str = None, wikiId: str = None) -> int:
        data = {
            "adminOpName": 114,
            "adminOpValue": {
                "featuredType": FeaturedTypes.UNFEATURE
            },
            "timestamp": int(time() * 1000)
        }

        if userId:
            response = await self.post(f"/x{self.comId}/s/user-profile/{userId}/admin", data)
            return response.status

        elif blogId:
            response = await self.post(f"/x{self.comId}/s/blog/{blogId}/admin", data)
            return response.status

        elif wikiId:
            response = await self.post(f"/x{self.comId}/s/item/{wikiId}/admin", data)
            return response.status

        elif chatId:
            response = await self.post(f"/x{self.comId}/s/chat/thread/{chatId}/admin", data)
            return response.status

        else: raise SpecifyType()
    
    async def hide(self, userId: str = None, chatId: str = None, blogId: str = None, wikiId: str = None, quizId: str = None, fileId: str = None, reason: str = None) -> int:
        data = {
            "adminOpName": 110,
            "adminOpValue": 9,
            "adminOpNote": {
                "content": reason
            },
            "timestamp": int(time() * 1000)
        }

        if userId:
            data["adminOpName"] = 18
            response = await self.post(f"/x{self.comId}/s/user-profile/{userId}/admin", data)
            return response.status

        elif blogId:
            response = await self.post(f"/x{self.comId}/s/blog/{blogId}/admin", data)
            return response.status

        elif quizId:
            response = await self.post(f"/x{self.comId}/s/blog/{quizId}/admin", data)
            return response.status

        elif wikiId:
            response = await self.post(f"/x{self.comId}/s/item/{wikiId}/admin", data)
            return response.status

        elif chatId:
            response = await self.post(f"/x{self.comId}/s/chat/thread/{chatId}/admin", data)
            return response.status

        elif fileId:
            response = await self.post(f"/x{self.comId}/s/shared-folder/files/{fileId}/admin", data)
            return response.status

        else: raise SpecifyType()

    async def unhide(self, userId: str = None, chatId: str = None, blogId: str = None, wikiId: str = None, quizId: str = None, fileId: str = None, reason: str = None) -> int:
        data = {
            "adminOpName": 110,
            "adminOpValue": 0,
            "adminOpNote": {
                "content": reason
            },
            "timestamp": int(time() * 1000)
        }

        if userId:
            data["adminOpName"] = 19
            response = await self.post(f"/x{self.comId}/s/user-profile/{userId}/admin", data)
            return response.status

        elif blogId:
            response = await self.post(f"/x{self.comId}/s/blog/{blogId}/admin", data)
            return response.status

        elif quizId:
            response = await self.post(f"/x{self.comId}/s/blog/{quizId}/admin", data)
            return response.status

        elif wikiId:
            response = await self.post(f"/x{self.comId}/s/item/{wikiId}/admin", data)
            return response.status

        elif chatId:
            response = await self.post(f"/x{self.comId}/s/chat/thread/{chatId}/admin", data)
            return response.status

        elif fileId:
            response = await self.post(f"/x{self.comId}/s/shared-folder/files/{fileId}/admin", data)
            return response.status

        else: raise SpecifyType()

    async def edit_titles(self, userId: str, titles: list, colors: list) -> int:
        titles = [{"title": titles, "color": colors} for titles, colors in zip(titles, colors)]

        data = {
            "adminOpName": 207,
            "adminOpValue": {
                "titles": titles
            },
            "timestamp": int(time() * 1000)
        }

        response = await self.post(f"/x{self.comId}/s/user-profile/{userId}/admin", data)
        return response.status

    # TODO : List all warning texts
    async def warn(self, userId: str, reason: str = None) -> int:
        data = {
            "uid": userId,
            "title": "Custom",
            "content": reason,
            "attachedObject": {
                "objectId": userId,
                "objectType": ObjectTypes.USER
            },
            "penaltyType": 0,
            "adminOpNote": {},
            "noticeType": 7,
            "timestamp": int(time() * 1000)
        }

        response = await self.post(f"/x{self.comId}/s/notice", data)
        return response.status

    # TODO : List all strike texts
    async def strike(self, userId: str, seconds: int, title: str = None, reason: str = None) -> int:
        data = {
            "uid": userId,
            "title": title,
            "content": reason,
            "attachedObject": {
                "objectId": userId,
                "objectType": ObjectTypes.USER
            },
            "penaltyType": 1,
            "penaltyValue": seconds,
            "adminOpNote": {},
            "noticeType": 4,
            "timestamp": int(time() * 1000)
        }

        response = await self.post(f"/x{self.comId}/s/notice", data)
        return response.status
    
    async def ban(self, userId: str, reason: str, banType: int = None) -> int:
        data = {
            "reasonType": banType,
            "note": {
                "content": reason
            },
            "timestamp": int(time() * 1000)
        }

        response = await self.post(f"/x{self.comId}/s/user-profile/{userId}/ban", data)
        return response.status

    async def unban(self, userId: str, reason: str) -> int:
        data = {
            "note": {
                "content": reason
            },
            "timestamp": int(time() * 1000)
        }

        response = await self.post(f"/x{self.comId}/s/user-profile/{userId}/unban", data)
        return response.status
    
    async def purchase(self, objectId: str, objectType: int, autoRenew: bool = False) -> int:
        data = {
            "v": 1,
            "objectId": objectId,
            "objectType": objectType,
            "paymentContext": {
                "discountStatus": 1, 
                "discountValue": 1, 
                "isAutoRenew": autoRenew
            },
            "timestamp": int(time() * 1000)
        }

        if self.profile.membershipStatus == 0:
            data["paymentContext"]["discountStatus"] = 0

        response = await self.post(f"/x{self.comId}/s/store/purchase", data)
        return response.status
    
    async def invite_to_vc(self, chatId: str, userId: str) -> int:
        data = {
            "uid": userId
        }

        response = await self.post(f"/x{self.comId}/s/chat/thread/{chatId}/vvchat-presenter/invite", data)
        return response.status

    async def add_poll_option(self, blogId: str, question: str) -> int:
        data = {
            "mediaList": None,
            "title": question,
            "type": 0,
            "timestamp": int(time() * 1000)
        }

        response = await self.post(f"/x{self.comId}/s/blog/{blogId}/poll/option", data)
        return response.status

    async def create_wiki_category(self, title: str, parentCategoryId: str, content: str = None) -> int:
        data = {
            "content": content,
            "icon": None,
            "label": title,
            "mediaList": None,
            "parentCategoryId": parentCategoryId,
            "timestamp": int(time() * 1000)
        }

        response = await self.post(f"/x{self.comId}/s/item-category", data)
        return response.status

    async def create_shared_folder(self, title: str) -> int:
        data = {
            "title": title,
            "timestamp": int(time() * 1000)
        }

        response = await self.post(f"/x{self.comId}/s/shared-folder/folders", data)
        return response.status

    async def submit_to_wiki(self, wikiId: str, message: str) -> int:
        data = {
            "message": message,
            "itemId": wikiId,
            "timestamp": int(time() * 1000)
        }

        response = await self.post(f"/x{self.comId}/s/knowledge-base-request", data)
        return response.status

    async def accept_wiki_request(self, requestId: str, destinationCategoryIdList: list) -> int:
        data = {
            "destinationCategoryIdList": destinationCategoryIdList,
            "actionType": "create",
            "timestamp": int(time() * 1000)
        }

        response = await self.post(f"/x{self.comId}/s/knowledge-base-request/{requestId}/approve", data)
        return response.status

    async def apply_avatar_frame(self, avatarId: str, applyToAll: bool = True) -> int:
        data = {
            "frameId": avatarId,
            "applyToAll": 0,
            "timestamp": int(time() * 1000)
        }

        if applyToAll:
            data["applyToAll"] = 1

        response = await self.post(f"/x{self.comId}/s/avatar-frame/apply", data)
        return response.status
    
    async def apply_chat_bubble(self, bubbleId: str, chatId: str, applyToAll: bool = False) -> int:
        data = {
            "applyToAll": 0,
            "bubbleId": bubbleId,
            "threadId": chatId,
            "timestamp": int(time() * 1000)
        }

        if applyToAll:
            data["applyToAll"] = 1

        response = await self.post(f"/x{self.comId}/s/chat/thread/apply-bubble", data)
        return response.status

    async def reject_wiki_request(self, requestId: str) -> int:
        response = await self.post(f"/x{self.comId}/s/knowledge-base-request/{requestId}/reject")
        return response.status

    # TODO : Finish this
    async def get_wiki_submissions(self, start: int = 0, size: int = 25) -> dict:
        response = await self.get(f"/x{self.comId}/s/knowledge-base-request?type=all&start={start}&size={size}")
        return (await response.json())["knowledgeBaseRequestList"]
    
    # Live Layer (i dont enf this)
    async def get_live_layer(self) -> dict:
        response = await self.get(f"/x{self.comId}/s/live-layer/homepage?v=2")
        return (await response.text())["liveLayerList"]
    
    async def get_online_users(self, start: int = 0, size: int = 25) -> List[UserProfile]:
        response = await self.get(f"/x{self.comId}/s/live-layer?topic=ndtopic:x{self.comId}:online-members&start={start}&size={size}")
        return list(map(lambda o: UserProfile(**o), (await response.json())["userProfileList"]))
    
    async def get_online_users_count(self) -> int:
        response = await self.get(f"/x{self.comId}/s/live-layer?topic=ndtopic:x{self.comId}:online-members&start=0&size=1")
        return (await response.json())["userProfileCount"]
    
    async def get_public_chats(self, start: int = 0, size: int = 25) -> List[Thread]:
        response = await self.get(f"/x{self.comId}/s/live-layer/public-chats?start={start}&size={size}")
        return list(map(lambda o: UserProfile(**o), (await response.json())["threadList"]))
    
    async def get_chatting_users(self, start: int = 0, size: int = 25) -> List[UserProfile]:
        response = await self.get(f"/x{self.comId}/s/live-layer?topic=ndtopic:x{self.comId}:users-chatting&start={start}&size={size}")
        return list(map(lambda o: UserProfile(**o), (await response.json())["userProfileList"]))
    
    async def get_chatting_users_count(self) -> int:
        response = await self.get(f"/x{self.comId}/s/live-layer?topic=ndtopic:x{self.comId}:users-chatting&start=0&size=1")
        return (await response.json())["userProfileCount"]

    async def get_live_chats(self, start: int = 0, size: int = 25) -> List[Thread]:
        response = await self.get(f"/x{self.comId}/s/live-layer/public-live-chats?start={start}&size={size}")
        return list(map(lambda o: Thread(**o), (await response.json())["threadList"]))
    
    async def get_live_chatting_users(self, start: int = 0, size: int = 25) -> List[UserProfile]:
        response = await self.get(f"/x{self.comId}/s/live-layer?topic=ndtopic:x{self.comId}:users-live-chatting&start={start}&size={size}")
        return list(map(lambda o: UserProfile(**o), (await response.json())["userProfileList"]))
    
    async def get_live_chatting_users_count(self) -> int:
        response = await self.get(f"/x{self.comId}/s/live-layer?topic=ndtopic:x{self.comId}:users-live-chatting&start=0&size=1")
        return (await response.json())["userProfileCount"]

    async def get_playing_quizzes(self, start: int = 0, size: int = 25) -> List[Blog]:
        response = await self.get(f"/x{self.comId}/s/live-layer/quizzes?start={start}&size={size}")
        return list(map(lambda o: Blog(**o), (await response.json())["blogList"]))
    
    async def get_playing_quizzes_users(self, start: int = 0, size: int = 25) -> List[UserProfile]:
        response = await self.get(f"/x{self.comId}/s/live-layer?topic=ndtopic:x{self.comId}:users-playing-quizzes&start={start}&size={size}")
        return list(map(lambda o: UserProfile(**o), (await response.json())["userProfileList"]))
    
    async def get_playing_quizzes_users_count(self) -> int:
        response = await self.get(f"/x{self.comId}/s/live-layer?topic=ndtopic:x{self.comId}:users-playing-quizzes&start=0&size=1")
        return (await response.json())["userProfileCount"]

    async def get_browsing_blogs(self, start: int = 0, size: int = 25) -> List[Blog]:
        response = await self.get(f"/x{self.comId}/s/live-layer/blogs?start={start}&size={size}")
        return list(map(lambda o: Blog(**o), (await response.json())["blogList"]))
    
    async def get_browsing_blogs_users(self, start: int = 0, size: int = 25) -> List[UserProfile]:
        response = await self.get(f"/x{self.comId}/s/live-layer?topic=ndtopic:x{self.comId}:users-browsing-blogs&start={start}&size={size}")
        return list(map(lambda o: UserProfile(**o), (await response.json())["userProfileList"]))
    
    async def get_browsing_blogs_users_count(self) -> int:
        response = await self.get(f"/x{self.comId}/s/live-layer?topic=ndtopic:x{self.comId}:users-browsing-blogs&start=0&size=1")
        return (await response.json())["userProfileCount"]

    async def get_blog_users(self, blogId: str, start: int = 0, size: int = 25):
        response = await self.get(f"/x{self.comId}/s/live-layer?topic=ndtopic%3Ax{self.comId}%3Ausers-browsing-blog-at%3A{blogId}&start={start}&size={size}")
        return list(map(lambda o: UserProfile(**o), (await response.json())["userProfileList"]))
    
    # Live Layer (i dont enf this)

    async def activate_bubble(self, bubbleId: str) -> int:
        response = await self.post(f"/x{self.comId}/s/chat/chat-bubble/{bubbleId}/activate")
        return response.status

    async def deactivate_bubble(self, bubbleId: str) -> int:
        response = await self.post(f"/x{self.comId}/s/chat/chat-bubble/{bubbleId}/deactivate")
        return response.status    
    
    async def get_chat_bubbles(self, chatId: str, start: int = 25, size: int = 25) -> List[ChatBubble]:
        response = await self.get(f"/x{self.comId}/s/chat/chat-bubble?type=all-my-bubbles?threadId={chatId}?start={start}?size={size}")
        return list(map(lambda o: ChatBubble(**o), (await response.json())["chatBubbleList"]))
    
    async def get_chat_bubble(self, bubbleId: str) -> ChatBubble:
        response = await self.get(f"/x{self.comId}/s/chat/chat-bubble/{bubbleId}")
        return ChatBubble(**(response.text)["chatBubble"])

    async def get_chat_bubble_templates(self, start: int = 0, size: int = 25) -> List[ChatBubble]:
        response = await self.get(f"/x{self.comId}/s/chat/chat-bubble/templates?start={start}&size={size}")
        return list(map(lambda o: ChatBubble(**o), (await response.json())["templateList"]))
    
    async def generate_chat_bubble(self, bubble: BinaryIO = None, 
            templateId: str = "949156e1-cc43-49f0-b9cf-3bbbb606ad6e") -> ChatBubble:
        response = await self.post(f"/x{self.comId}/s/chat/chat-bubble/templates/{templateId}/generate", bubble)
        return ChatBubble(**(response.text)["chatBubble"])
    
    async def edit_chat_bubble(self, bubbleId: str, bubble: BinaryIO) -> ChatBubble:
        response = await self.post(f"/x{self.comId}/s/chat/chat-bubble/{bubbleId}", bubble)
        return ChatBubble(**(response.text)["chatBubble"])

    async def send_active_object(self, timers: List[dict], timezone: int = 0, flags: int = 0):
        data = {
            "userActiveTimeChunkList": timers, 
            "optInAdsFlags": flags, 
            "timestamp": int(time() * 1000), 
            "timezone": timezone
        }

        data = json_minify(dumps(data))
        response = await self.post(f"/x{self.comId}/s/community/stats/user-active-time", data=data)
        return response.status




__title__ = 'Amino.ed'
__author__ = 'Alert Aigul'
__license__ = 'MIT'
__copyright__ = 'Copyright 2020-2022 Alert'
__version__ = '2.3.4'

from asyncio import get_event_loop
from asyncio.events import AbstractEventLoop
from requests import get

from .global_client import Client
from .community_client import CommunityClient
from .utils import exceptions, models, types, helpers
from .websocket import WebSocketClient


def run_with_client(event_loop: AbstractEventLoop = None, deviceId: str = None):
    async def start(loop, callback):
        async with Client(loop, deviceId) as client:
            await callback(client)

    def _start(callback):
        loop = event_loop or get_event_loop()
        loop.run_until_complete(start(loop, callback))
    return _start


__newest__ = get("https://pypi.python.org/pypi/Amino.ed/json").json()["info"]["version"]

if __version__ != __newest__:
    print(f"New version available: {__newest__} (Using {__version__})")



from setuptools import setup, find_packages


with open("README.md", "r") as stream:
    long_description = stream.read()

setup(
    name="Amino.ed",
    version="2.3.4",
    url="https://github.com/Alert-Aigul/Amino.ed",
    download_url="https://github.com/Alert-Aigul/Amino.ed/archive/refs/heads/main.zip",
    license="MIT",
    author="Alert Aigul",
    author_email="alertaigul@gmail.com",
    description="A library to create Amino bots.",
    long_description=long_description,
    long_description_content_type="text/markdown",
    keywords=[
        "aminoapps",
        "amino-py",
        "amino",
        "amino-bot",
        "amino.py",
        "amino.ed",
        "amino-ed",
        "narvii",
        "api",
        "python",
        "python3",
        "python3.x",
        "official",
        "alert",
        "fix",
        "ed"
    ],
    install_requires=[
        "requests",
        "setuptools",
        "six",
        "aiohttp",
        "ujson",
        "requests",
        "eventemitter",
        "typing",
        "pydantic"
    ],
    setup_requires=[
        "wheel"
    ],
    classifiers=[
        "Development Status :: 4 - Beta",
        "Intended Audience :: Developers",
        "Programming Language :: Python :: 3.6",
    ],
    python_requires=">=3.6",
    packages=find_packages()
)



from pydantic import BaseModel, Field
from typing import List, Optional, Any, Union, Dict
from datetime import datetime


class RestrictionInfo(BaseModel):
    discountStatus:    Optional[int]
    ownerUid:          Optional[str]
    ownerType:         Optional[int]
    restrictType:      Optional[int]
    restrictValue:     Optional[int]
    availableDuration: Optional[int]
    discountValue:     Optional[int]


class AvatarFrame(BaseModel):
    class Config(BaseModel):
        name:                Optional[str]
        version:             Optional[int]
        userIconBorderColor: Optional[str]
        avatarFramePath:     Optional[str]
        id:                  Optional[str]
        moodColor:           Optional[str]

    class OwnershipInfo(BaseModel):
        isAutoRenew:     Optional[bool]
        expiredTime:     Optional[datetime]
        createdTime:     Optional[datetime]
        ownershipStatus: Optional[int]

    restrictionInfo:     Optional[RestrictionInfo]
    frameUrl:            Optional[str]
    icon:                Optional[str]
    frameId:             Optional[str]
    ownershipStatus:     Optional[int]
    extensions:          Optional[Dict]
    uid:                 Optional[str]
    version:             Optional[int]
    isGloballyAvailable: Optional[bool]
    ownershipInfo:       Optional[OwnershipInfo]
    frameType:           Optional[int]
    md5:                 Optional[str]
    config:              Optional[Config]
    status:              Optional[int]
    name:                Optional[str]
    availableNdcIds:     Optional[List[int]]
    modifiedTime:        Optional[datetime]
    createdTime:         Optional[datetime]
    isNew:               Optional[bool]
    resourceUrl:         Optional[str]


class Account(BaseModel):
    class Extensions(BaseModel):
        class DeviceInfo(BaseModel):
            lastClientType: Optional[int]

        deviceInfo:      Optional[DeviceInfo]
        popupConfig:     Optional[Dict]
        contentLanguage: Optional[str]
        adsFlags:        Optional[str]
        adsLevel:        Optional[int]
        avatarFrameId:   Optional[str]
        adsEnabled:      Optional[bool]

    class AdvancedSettings(BaseModel):
        analyticsEnabled: Optional[int]

    extensions:            Optional[Extensions]
    advancedSettings:      Optional[AdvancedSettings]
    username:              Optional[str]
    status:                Optional[int]
    uid:                   Optional[str]
    modifiedTime:          Optional[datetime]
    twitterID:             Optional[Any]
    activation:            Optional[int]
    phoneNumberActivation: Optional[int]
    emailActivation:       Optional[int]
    appleID:               Optional[Any]
    facebookID:            Optional[Any]
    nickname:              Optional[str]
    googleID:              Optional[str]
    icon:                  Optional[str]
    securityLevel:         Optional[str]
    phoneNumber:           Optional[str]
    membership:            Optional[Any]
    role:                  Optional[int]
    aminoIdEditable:       Optional[bool]
    aminoId:               Optional[str]
    createdTime:           Optional[datetime]
    email:                 Optional[str]


class UserProfile(BaseModel):
    class Settings(BaseModel):
        onlineStatus: Optional[int]

    class Extensions(BaseModel):
        class Style(BaseModel):
            backgroundMediaList: Optional[List[List]]

        style: Optional[Style]

    class InfluencerInfo(BaseModel):
        fansCount:   Optional[int]
        monthlyFee:  Optional[int]
        pinned:      Optional[bool]
        createdTime: Optional[datetime]

    influencerInfo:          Optional[InfluencerInfo]
    avatarFrame:             Optional[AvatarFrame]
    settings:                Optional[Settings]
    status:                  Optional[int]
    moodSticker:             Optional[Any]
    itemsCount:              Optional[int]
    consecutiveCheckInDays:  Optional[str]
    userId:                  Optional[str] = Field(alias="uid")
    modifiedTime:            Optional[datetime]
    followingStatus:         Optional[int]
    onlineStatus:            Optional[int]
    accountMembershipStatus: Optional[int]
    isGlobal:                Optional[bool]
    avatarFrameId:           Optional[str]
    reputation:              Optional[int]
    postsCount:              Optional[int]
    followersCount:          Optional[int]
    nickname:                Optional[str]
    icon:                    Optional[str]
    isVerified:              Optional[bool]
    visitorsCount:           Optional[int]
    mood:                    Optional[Any]
    level:                   Optional[int]
    pushEnabled:             Optional[bool]
    membershipStatus:        Optional[int]
    content:                 Optional[str]
    followedCount:           Optional[int]
    role:                    Optional[int]
    commentsCount:           Optional[int]
    aminoId:                 Optional[str]
    comId:                   Optional[int] = Field(alias="ndcId")
    createdTime:             Optional[datetime]
    visitPrivacy:            Optional[int]
    storiesCount:            Optional[int]
    blogsCount:              Optional[int]


class Auth(BaseModel):
    auid:        Optional[str]
    account:     Optional[Account]
    sid:         Optional[str]
    secret:      Optional[str]
    userProfile: Optional[UserProfile]
    deviceId:    Optional[str]


class Community(BaseModel):
    class AdvancedSettings(BaseModel):
        class RankingTableItem(BaseModel):
            title:      Optional[str]
            level:      Optional[int]
            reputation: Optional[int]
            id:         Optional[int]

        class NewsFeedPage(BaseModel):
            status: Optional[int]
            type:   Optional[int]

        rankingTable:                    Optional[List[RankingTableItem]]
        frontPageLayout:                 Optional[int]
        leaderboardStyle:                Optional[Dict]
        newsfeedPages:                   Optional[List[NewsFeedPage]]
        facebookAppIdList:               Optional[Any]
        hasPendingReviewRequest:         Optional[bool]
        joinedBaselineCollectionIdList:  Optional[List]
        defaultRankingTypeInLeaderboard: Optional[int]
        welcomeMessageText:              Optional[Any]
        pollMinFullBarVoteCount:         Optional[int]
        welcomeMessageEnabled:           Optional[Any]
        catalogEnabled:                  Optional[bool]

    class UserAddedTopic(BaseModel):
        class Style(BaseModel):
            backgroundColor: Optional[str]

        style:   Optional[Style]
        topicId: Optional[int]
        name:    Optional[str]

    class ThemePack(BaseModel):
        themePackHash:     Optional[str]
        themePackRevision: Optional[int]
        themePackUrl:      Optional[str]
        themeColor:        Optional[str]

    class Configuration(BaseModel):
        class Module(BaseModel):
            class Ranking(BaseModel):
                class Leaderboard(BaseModel):
                    type:    Optional[int]
                    id:      Optional[int]
                    enabled: Optional[bool]

                class Table(BaseModel):
                    id: Optional[int]
                    title:      Optional[str]
                    level:      Optional[int]
                    reputation: Optional[int]

                enabled:                Optional[bool]
                leaderboardList:        Optional[List[Leaderboard]]
                leaderboardEnabled:     Optional[bool]
                rankingTable:           Optional[List[Table]]
                defaultLeaderboardType: Optional[int]

            class Chat(BaseModel):
                class AvChat(BaseModel):
                    audioEnabled:         Optional[bool]
                    videoEnabled:         Optional[bool]
                    audio2Enabled:        Optional[bool]
                    screeningRoomEnabled: Optional[bool]

                class PublicChat(BaseModel):
                    class Privilege(BaseModel):
                        type:     Optional[int]
                        minLevel: Optional[int]

                    privilege: Optional[Privilege]
                    enabled:   Optional[bool]

                avChat:                Optional[AvChat]
                publicChat:            Optional[PublicChat]
                spamProtectionEnabled: Optional[bool]
                enabled:               Optional[bool]

            class ExternalContent(BaseModel):
                enabled: Optional[bool]

            class Post(BaseModel):
                class PostType(BaseModel):
                    class Type(BaseModel):
                        class Privilege(BaseModel):
                            type:     Optional[int]
                            minLevel: int = 0

                        privilege: Optional[Privilege]
                        enabled:   Optional[bool]

                    image:           Optional[Type]
                    quiz:            Optional[Type]
                    question:        Optional[Type]
                    blog:            Optional[Type]
                    publicChatRooms: Optional[Type]
                    poll:            Optional[Type]
                    screeningRoom:   Optional[Type]
                    catalogEntry:    Optional[Type]
                    story:           Optional[Type]
                    liveMode:        Optional[Type]
                    webLink:         Optional[Type]

                postType: Optional[PostType]
                enabled:  Optional[bool]

            class Influencer(BaseModel):
                enabled:          Optional[bool]
                maxVipMonthlyFee: Optional[int]
                minVipMonthlyFee: Optional[int]
                lock:             Optional[bool]
                maxVipNumbers:    Optional[int]

            class Catalog(BaseModel):
                class Privilege(BaseModel):
                    type: Optional[int]

                privilege:       Optional[Privilege]
                enabled:         Optional[bool]
                curationEnabled: Optional[bool]

            class SharedFolder(BaseModel):
                class AlbumManagePrivilege(BaseModel):
                    type:     Optional[int]
                    minLevel: Optional[int]

                class UploadPrivilege(BaseModel):
                    minLevel: Optional[int]
                    type:     Optional[int]

                uploadPrivilege:      Optional[UploadPrivilege]
                albumManagePrivilege: Optional[AlbumManagePrivilege]
                enabled:              Optional[bool]

            class Featured(BaseModel):
                enabled:               Optional[bool]
                lockMember:            Optional[bool]
                postEnabled:           Optional[bool]
                layout:                Optional[int]
                publicChatRoomEnabled: Optional[bool]
                memberEnabled:         Optional[bool]

            ranking: Optional[Ranking]
            chat:            Optional[Chat]
            externalContent: Optional[ExternalContent]
            post:            Optional[Post]
            influencer:      Optional[Influencer]
            catalog:         Optional[Catalog]
            sharedFolder:    Optional[SharedFolder]
            featured:        Optional[Featured]

        class General(BaseModel):
            class WelcomeMessage(BaseModel):
                enabled: Optional[bool]
                text:    Optional[str]

            welcomeMessage:                 Optional[WelcomeMessage]
            disableLiveLayerVisible:        Optional[bool]
            joinedTopicIdList:              Optional[List[int]]
            disableLocation:                Optional[bool]
            premiumFeatureEnabled:          Optional[bool]
            onlyAllowOfficialTag:           Optional[bool]
            videoUploadPolicy:              Optional[int]
            accountMembershipEnabled:       Optional[bool]
            joinedBaselineCollectionIdList: Optional[List]
            invitePermission:               Optional[int]
            hasPendingReviewRequest:        Optional[bool]
            disableLiveLayerActive:         Optional[bool]
            facebookAppIdList:              Optional[Any]

        class Appearance(BaseModel):
            class HomePage(BaseModel):
                class IdObj(BaseModel):
                    id: Optional[str]

                navigation: Optional[List[IdObj]]

            class LeftSidePanel(BaseModel):
                class Navigation(BaseModel):
                    class IdObj(BaseModel):
                        id: Optional[str]

                    level2: Optional[List[IdObj]]
                    level1: Optional[List[IdObj]]

                class Style(BaseModel):
                    iconColor: Optional[str]

                navigation: Optional[Navigation]
                style:      Optional[Style]

            homePage:      Optional[HomePage]
            leftSidePanel: Optional[LeftSidePanel]

        class Page(BaseModel):
            class DefaultObject(BaseModel):
                url:   Optional[str]
                alias: Optional[Any]
                id:    Optional[str]

            defaultList: Optional[List[DefaultObject]]
            customList:  Optional[List]

        module:     Optional[Module]
        appearance: Optional[Appearance]
        general:    Optional[General]
        page:       Optional[Page]

    icon:                               Optional[str]
    configuration:                      Optional[Configuration]
    communityHeadList:                  Optional[List[UserProfile]]
    status:                             Optional[int]
    probationStatus:                    Optional[int]
    influencerList:                     Optional[List[UserProfile]]
    ListedStatus:                       Optional[int]
    advancedSettings:                   Optional[AdvancedSettings]
    searchable:                         Optional[bool]
    communityHeat:                      Optional[int]
    endpoint:                           Optional[str]
    joinType:                           Optional[int]
    modifiedTime:                       Optional[datetime]
    userAddedTopicList:                 Optional[List[UserAddedTopic]]
    content:                            Optional[Any]
    comId:                              Optional[int] = Field(alias="ndcId")
    mediaList:                          Optional[Any]
    primaryLanguage:                    Optional[str]
    keywords:                           Optional[str]
    name:                               Optional[str]
    membersCount:                       Optional[int]
    isStandaloneAppDeprecated:          Optional[bool]
    isStandaloneAppMonetizationEnabled: Optional[bool]
    link:                               Optional[str]
    agent:                              Optional[UserProfile]
    themePack:                          Optional[ThemePack]
    activeInfo:                         Optional[Dict]
    templateId:                         Optional[int]
    tagline:                            Optional[str]
    createdTime:                        Optional[datetime]
    extensions:                         Optional[Any]
    promotionalMediaList:               Optional[List[List]]


class ChatBubble(BaseModel):
    class Config(BaseModel):
        status:               Optional[int]
        allowedSlots:         Optional[List[Dict]]
        name:                 Optional[str]
        contentInsets:        Optional[List[int]]
        zoomPoint:            Optional[List[int]]
        version:              Optional[int]
        linkColor:            Optional[str]
        slots:                Optional[List[Dict]]
        backgroundPath:       Optional[str]
        id:                   Optional[str]
        color:                Optional[str]
        previewBackgroundUrl: Optional[str]

    config:          Optional[Config]
    status:          Optional[int]
    resourceUrl:     Optional[str]
    name:            Optional[str]
    modifiedTime:    Optional[datetime]
    uid:             Optional[str]
    restrictionInfo: Optional[Any]
    coverImage:      Optional[str]
    isNew:           Optional[bool]
    bubbleType:      Optional[int]
    ownershipStatus: Optional[int]
    version:         Optional[int]
    backgroundImage: Optional[str]
    extensions:      Optional[Dict]
    deletable:       Optional[bool]
    templateId:      Optional[str]
    createdTime:     Optional[datetime]
    bubbleId:        Optional[str]
    bannerImage:     Optional[str]
    md5:             Optional[str]


class TipInfo(BaseModel):
    class TipOption(BaseModel):
        value: Optional[int]
        icon:  Optional[str]

    tipCustomOption: Optional[TipOption]
    tipOptionList:   Optional[List[TipOption]]
    tipMaxCoin:      Optional[int]
    tippersCount:    Optional[int]
    tippable:        Optional[bool]
    tipMinCoin:      Optional[int]
    tippedCoins:     Optional[float]


class Thread(BaseModel):
    class Topic(BaseModel):
        class Style(BaseModel):
            backgroundColor: Optional[str]

        status:                  Optional[int]
        isOfficial:              Optional[bool]
        style:                   Optional[Style]
        isAlias:                 Optional[bool]
        name:                    Optional[str]
        contentPoolId:           Optional[str]
        subTopicList:            Optional[List]
        coverImage:              Optional[str]
        aliasTopicList:          Optional[List]
        advancedCommunityStatus: Optional[int]
        increaseId:              Optional[int]
        visibility:              Optional[int]
        source:                  Optional[int]
        chatStatus:              Optional[int]
        topicId:                 Optional[int]
        storyId:                 Optional[str]
        scope:                   Optional[int]
        advancedStatus:          Optional[int]
        isLocked:                Optional[bool]
        tabList:                 Optional[List]
        objectMappingScore:      Optional[int]

    class Member(BaseModel):
        status:           Optional[int]
        userId:           Optional[str] = Field(alias="uid")
        membershipStatus: Optional[int]
        role:             Optional[int]
        nickname:         Optional[str]
        icon:             Optional[str]

    class LastMessage(BaseModel):
        userId:      Optional[str] = Field(alias="uid")
        type:        Optional[int]
        mediaType:   Optional[int]
        content:     Optional[str]
        messageId:   Optional[str]
        createdTime: Optional[datetime]
        isHidden:    Optional[bool]
        mediaValue:  Optional[Any]

    class Extensions(BaseModel):
        class ScreeningRoomPermission(BaseModel):
            action:  Optional[int]
            uidList: Optional[List]

        screeningRoomPermission:      Optional[ScreeningRoomPermission]
        viewOnly:                     Optional[bool]
        coHost:                       Optional[List[str]]
        language:                     Optional[str]
        membersCanInvite:             Optional[bool]
        bm:                           Optional[List]
        creatoruserId:                Optional[str] = Field(alias="uid")
        visibility:                   Optional[int]
        bannedMemberUidList:          Optional[List[str]]
        lastMembersSummaryUpdateTime: Optional[int]
        fansOnly:                     Optional[bool]
        announcement:                 Optional[str]
        channelType:                  Optional[int]
        pinAnnouncement:              Optional[bool]
        vvChatJoinType:               Optional[int]

    author:             Optional[UserProfile]
    extensions:         Optional[Extensions]
    lastMessageSummary: Optional[LastMessage]
    tipInfo:            Optional[TipInfo]
    userAddedTopicList: Optional[Topic]
    userId:             Optional[str] = Field(alias="uid")
    membersSummary:     Optional[List[Member]]
    membersQuota:       Optional[int]
    chatId:             Optional[str] = Field(alias="threadId")
    keywords:           Optional[str]
    membersCount:       Optional[int]
    strategyInfo:       Optional[Any]
    isPinned:           Optional[bool]
    title:              Optional[str]
    membershipStatus:   Optional[int]
    content:            Optional[str]
    needHidden:         Optional[bool]
    alertOption:        Optional[int]
    lastReadTime:       Optional[datetime]
    type:               Optional[int]
    status:             Optional[int]
    publishToGlobal:    Optional[int]
    modifiedTime:       Optional[datetime]
    condition:          Optional[int]
    icon:               Optional[str]
    latestActivityTime: Optional[datetime]
    comId:              Optional[int] = Field(alias="ndcId")
    createdTime:        Optional[datetime]
    chatBubbles:        Optional[Dict[str, ChatBubble]]


class Sticker(BaseModel):
    status:              Optional[int]
    iconV2:              Optional[str]
    name:                Optional[str]
    stickerId:           Optional[str]
    smallIconV2:         Optional[Any]
    smallIcon:           Optional[Any]
    stickerCollectionId: Optional[Any]
    mediumIcon:          Optional[Any]
    extensions:          Optional[Dict]
    usedCount:           Optional[int]
    mediumIconV2:        Optional[Any]
    createdTime:         Optional[datetime]
    icon:                Optional[str]


class BaseMessage(BaseModel):
    class Extensions(BaseModel):
        sticker:           Optional[Sticker]
        originalStickerId: Optional[str]

    extensions:        Optional[Extensions]
    includedInSummary: Optional[bool]
    author:            Optional[UserProfile]
    isHidden:          Optional[bool]
    chatBubble:        Optional[ChatBubble]
    messageId:         Optional[str]
    mediaType:         Optional[int]
    content:           Optional[str]
    chatBubbleId:      Optional[str]
    clientRefId:       Optional[int]
    chatId:            Optional[str] = Field(alias="threadId")
    createdTime:       Optional[datetime]
    chatBubbleVersion: Optional[int]
    type:              Optional[int]
    mediaValue:        Optional[str]
    base_reply:        Optional['BaseMessage']
    nextPageToken:     Optional[str]
    prevPageToken:     Optional[str]

    @property
    def reply(self):
        if self.base_reply is None:
            self.base_reply = BaseMessage(**{})
        return self.base_reply

class Message(BaseMessage):
    def __init__(self, **data) -> None:
        replies: List[BaseMessage] = []
        replies.append(BaseMessage(**data))

        while (val := (data.get('extensions') or {}).get("replyMessage")) is not None:
            replies.append(BaseMessage(**data))
            data = val if data is not None else {}

        replies[-1] = BaseMessage(**replies[-1].dict())

        for i in range(len(replies) - 1, 0, -1):
            message = replies[i-1].dict()
            replies[i - 1] = BaseMessage(**message, rep=replies[i])
        super().__init__(**replies[0].dict())

Message.update_forward_refs()
BaseMessage.update_forward_refs()


class Blog(BaseModel):
    class QuizQuestion(BaseModel):
        class Extensions(BaseModel):
            class QuestionOpt(BaseModel):
                optId: Optional[str]
                qhash: Optional[str]
                title: Optional[str]

            quizQuestionOptList: Optional[List[QuestionOpt]]
            __disabledLevel__: Optional[int]

        status:         Optional[int]
        parentType:     Optional[int]
        extensions:     Optional[Extensions]
        title:          Optional[str]
        createdTime:    Optional[datetime]
        quizQuestionId: Optional[str]
        parentId:       Optional[str]
        mediaList:      Optional[List[List]]

    class Extensions(BaseModel):
        class Style(BaseModel):
            coverMediaIndexList: Optional[Any]
            backgroundMediaList: Optional[List[List]]

        style:                   Optional[Style]
        quizPlayedTimes:         Optional[int]
        quizTotalQuestionCount:  Optional[int]
        quizTrendingTimes:       Optional[int]
        quizLastAddQuestionTime: Optional[int]
        fansOnly:                Optional[bool]

    globalVotesCount:      Optional[int]
    globalVotedValue:      Optional[int]
    votedValue:            Optional[int]
    keywords:              Optional[str]
    mediaList:             Optional[List[List]]
    style:                 Optional[int]
    totalQuizPlayCount:    Optional[int]
    title:                 Optional[str]
    tipInfo:               Optional[TipInfo]
    contentRating:         Optional[int]
    content:               Optional[str]
    needHidden:            Optional[bool]
    guestVotesCount:       Optional[int]
    type:                  Optional[int]
    status:                Optional[int]
    globalCommentsCount:   Optional[int]
    modifiedTime:          Optional[datetime]
    quizQuestionList:      Optional[List[QuizQuestion]]
    widgetDisplayInterval: Optional[Any]
    totalPollVoteCount:    Optional[int]
    blogId:                Optional[str]
    viewCount:             Optional[int]
    language:              Optional[str]
    author:                Optional[UserProfile]
    extensions:            Optional[Extensions]
    votesCount:            Optional[int]
    comId:                 Optional[int] = Field(alias="ndcId")
    createdTime:           Optional[datetime]
    endTime:               Optional[Any]
    commentsCount:         Optional[int]
    nextPageToken:         Optional[str]
    prevPageToken:         Optional[str]


class Wiki(BaseModel):
    class Extensions(BaseModel):
        fansOnly: Optional[bool]

    globalVotesCount:    Optional[int]
    globalVotedValue:    Optional[int]
    votedValue:          Optional[int]
    keywords:            Optional[str]
    mediaList:           Optional[List[List]]
    style:               Optional[int]
    author:              Optional[UserProfile]
    tipInfo:             Optional[TipInfo]
    contentRating:       Optional[int]
    label:               Optional[str]
    content:             Optional[str]
    needHidden:          Optional[bool]
    guestVotesCount:     Optional[int]
    status:              Optional[int]
    globalCommentsCount: Optional[int]
    modifiedTime:        Optional[datetime]
    itemId:              Optional[str]
    extensions:          Optional[Extensions]
    votesCount:          Optional[int]
    comId:               Optional[int] = Field(alias="ndcId")
    createdTime:         Optional[datetime]
    commentsCount:       Optional[int]


class Comment(BaseModel):
    modifiedTime:     Optional[datetime]
    comId:            Optional[int] = Field(alias="ndcId")
    votedValue:       Optional[int]
    parentType:       Optional[int]
    commentId:        Optional[str]
    parentComId:      Optional[int] = Field(alias="parentNdcId")
    mediaList:        Optional[List[List]]
    votesSum:         Optional[int]
    author:           Optional[UserProfile]
    content:          Optional[str]
    extensions:       Optional[Dict]
    parentId:         Optional[str]
    createdTime:      Optional[datetime]
    subcommentsCount: Optional[int]
    type:             Optional[int]


class Link(BaseModel):
    objectId:   Optional[str]
    targetCode: Optional[int]
    comId:      Optional[int] = Field(alias="ndcId")
    fullPath:   Optional[str]
    shortCode:  Optional[str]
    objectType: Optional[int]
    community:  Optional[Community]


class Wallet(BaseModel):
    class AdsVideoStats(BaseModel):
        watchVideoMaxCount:     Optional[int]
        nextWatchVideoInterval: Optional[float]
        watchedVideoCount:      Optional[int]
        canWatchVideo:          Optional[bool]
        canEarnedCoins:         Optional[int]
        canNotWatchVideoReason: Optional[str]

    adsVideoStats:           Optional[AdsVideoStats]
    totalBusinessCoins:      Optional[int]
    totalBusinessCoinsFloat: Optional[float]
    totalCoinsFloat:         Optional[float]
    adsEnabled:              Optional[bool]
    adsFlags:                Optional[int]
    totalCoins:              Optional[int]
    businessCoinsEnabled:    Optional[bool]


class Transaction(BaseModel):
    class ExtData(BaseModel):
        icon: Optional[str]
        subtitle: Optional[str]
        objectDeeplinkUrl: Optional[str]
        description: Optional[str]

    extData:           Optional[ExtData]
    userId:            Optional[str] = Field(alias="uid")
    totalCoins:        Optional[int]
    originCoins:       Optional[int]
    bonusCoins:        Optional[int]
    changedCoins:      Optional[int]
    taxCoins:          Optional[int]
    sourceType:        Optional[int]
    taxCoinsFloat:     Optional[float]
    bonusCoinsFloat:   Optional[float]
    totalCoinsFloat:   Optional[float]
    createdTime:       Optional[datetime]
    changedCoinsFloat: Optional[float]
    isPositive:        Optional[bool]
    originCoinsFloat:  Optional[float]


class Membership(BaseModel):
    userId:           Optional[str] = Field(alias="uid")
    paymentType:      Optional[int]
    expiredTime:      Optional[datetime]
    renewedTime:      Optional[datetime]
    modifiedTime:     Optional[datetime]
    createdTime:      Optional[datetime]
    isAutoRenew:      Optional[bool]
    membershipStatus: Optional[int]


class InviteCode(BaseModel):
    status:       Optional[int]
    duration:     Optional[int]
    invitationId: Optional[str]
    link:         Optional[str]
    modifiedTime: Optional[datetime]
    comId:        Optional[int] = Field(alias="ndcId")
    createdTime:  Optional[datetime]
    inviteCode:   Optional[str]


class StickerCollection(BaseModel):
    class Extensions(BaseModel):
        class OriginalCommunity(BaseModel):
            status:   Optional[int]
            icon:     Optional[str]
            endpoint: Optional[str]
            name:     Optional[str]
            comId:    Optional[int] = Field(alias="ndcId")

        iconSourceStickerId: Optional[str]
        originalAuthor:      Optional[UserProfile]
        originalCommunity:   Optional[OriginalCommunity]

    status:              Optional[int]
    isActivated:         Optional[bool]
    collectionType:      Optional[int]
    userId:              Optional[str] = Field(alias="uid")
    modifiedTime:        Optional[datetime]
    isNew:               Optional[bool]
    bannerUrl:           Optional[str]
    smallIcon:           Optional[str]
    stickersCount:       Optional[int]
    ownershipStatus:     Optional[Any]
    usedCount:           Optional[int]
    availableNdcIds:     Optional[List]
    icon:                Optional[str]
    name:                Optional[str]
    collectionId:        Optional[str]
    description:         Optional[str]
    author:              Optional[UserProfile]
    extensions:          Optional[Extensions]
    createdTime:         Optional[datetime]
    isGloballyAvailable: Optional[bool]
    restrictionInfo:     Optional[RestrictionInfo]


class Lottery(BaseModel):
    class LotteryLog(BaseModel):
        awardValue:  Optional[int]
        parentType:  Optional[int]
        objectId:    Optional[str]
        parentId:    Optional[str]
        createdTime: Optional[datetime]
        awardType:   Optional[int]
        refObject:   Optional[Dict]
        objectType:  Optional[int]

    wallet:     Optional[Wallet]
    lotteryLog: Optional[LotteryLog]


class Achievement(BaseModel):
    secondsSpentOfLast24Hours: Optional[float]
    secondsSpentOfLast7Days:   Optional[float]
    numberOfMembersCount:      Optional[int]
    numberOfPostsCreated:      Optional[int]


class CheckIn(BaseModel):
    class CheckInHistory(BaseModel):
        joinedTime:             Optional[int]
        stopTime:               Optional[int]
        consecutiveCheckInDays: Optional[int]
        streakRepairCoinCost:   Optional[int]
        startTime:              Optional[int]
        hasCheckInToday:        Optional[bool]
        streakRepairWindowSize: Optional[int]
        hasAnyCheckIn:          Optional[bool]
        history:                Optional[str]

    consecutiveCheckInDays:    Optional[int]
    canPlayLottery:            Optional[bool]
    earnedReputationPoint:     Optional[int]
    additionalReputationPoint: Optional[int]
    checkInHistory:            Optional[CheckInHistory]
    userProfile:               Optional[UserProfile]


class TippedUserSummary(BaseModel):
    tipper:           Optional[UserProfile]
    lastTippedTime:   Optional[datetime]
    totalTippedCoins: Optional[float]
    lastThankedTime:  Optional[datetime]


class Bookmark(BaseModel):
    refObjectType:  Optional[int]
    bookmarkedTime: Optional[datetime]
    refObjectId:    Optional[str]
    refObject:      Optional[Union[Blog, Wiki]]


class QuizRanking(BaseModel):
    highestMode:    Optional[int]
    modifiedTime:   Optional[datetime]
    isFinished:     Optional[bool]
    hellIsFinished: Optional[bool]
    highestScore:   Optional[int]
    beatRate:       Optional[Any]
    lastBeatRate:   Optional[Any]
    totalTimes:     Optional[int]
    latestScore:    Optional[int]
    author:         Optional[UserProfile]


class BlogCategory(BaseModel):
    blogsCount:   Optional[int]
    status:       Optional[int]
    type:         Optional[int]
    modifiedTime: Optional[datetime]
    label:        Optional[str]
    style:        Optional[int]
    categoryId:   Optional[str]
    createdTime:  Optional[datetime]
    position:     Optional[int]
    icon:         Optional[str]
    content:      Optional[str]


class VcReputation(BaseModel):
    participantCount: Optional[int]
    totalReputation:  Optional[int]
    duration:         Optional[int]


class AdminLog(BaseModel):
    class Author(BaseModel):
        icon:     Optional[str]
        status:   Optional[int]
        role:     Optional[int]
        nickname: Optional[str]
        uid:      Optional[str]
    
    objectUrl:       Optional[str]
    operationName:   Optional[str]
    createdTime:     Optional[datetime]
    referTicketId:   Optional[str]
    extData:         Optional[Dict]
    objectId:        Optional[str]
    moderationLevel: Optional[int]
    refObject:       Optional[Dict]
    author:          Optional[Author]
    objectType:      Optional[int]
    operation:       Optional[int]
    operationDetail: Optional[str]
    comId:           Optional[int] = Field(alias="ndcId")
    operationLevel:  Optional[str]
    logId:           Optional[str]


class StoreItem(BaseModel):
    class ItemBasicInfo(BaseModel):
        icon: Optional[str]
        name: Optional[str]

    refObjectType:       Optional[int]
    refObjectId:         Optional[str]
    createdTime:         Optional[datetime]
    itemBasicInfo:       Optional[ItemBasicInfo]
    itemRestrictionInfo: Optional[RestrictionInfo]
    refObject:           Optional[Union[ChatBubble, AvatarFrame, StickerCollection]]


class StoreSections(BaseModel):
    name:                 Optional[str]
    sectionGroupId:       Optional[str]
    allItemsCount:        Optional[int]
    icon:                 Optional[str]
    storeSectionId:       Optional[str]
    previewStoreItemList: Optional[StoreItem]

class Event(BaseModel):
    comId:            Optional[int] = Field(alias="ndcId")
    message:          Optional[Message] = Field(alias="chatMessage")
    alertOption:      Optional[int]
    membershipStatus: Optional[int]
